# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2015 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the 'Global numerical parameters' page.

This module contains the following classes:
- GlobalNumericalParametersAdvancedOptionsDialogView
- GlobalNumericalParametersView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import os, sys, string, types
import logging

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import GuiParam
from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator, from_qvariant
from code_saturne.Pages.GlobalNumericalParameters import Ui_GlobalNumericalParameters
from code_saturne.Pages.GlobalNumericalParametersAdvancedOptionsDialog import Ui_GlobalNumericalParametersAdvancedOptionsDialog
from code_saturne.model.GlobalNumericalParametersModel import GlobalNumericalParametersModel
from code_saturne.model.MainFieldsModel import MainFieldsModel

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("GlobalNumericalParametersView")
log.setLevel(GuiParam.DEBUG)


#-------------------------------------------------------------------------------
# GlobalNumericalParametersAdvancedOptionsDialogView class
#-------------------------------------------------------------------------------

class GlobalNumericalParametersAdvancedOptionsDialogView(QDialog, Ui_GlobalNumericalParametersAdvancedOptionsDialog):
    """
    Advanced global numerical parameters layout.
    """
    def __init__(self, parent, case, default):
        """
        Constructor
        """
        QDialog.__init__(self, parent)

        Ui_GlobalNumericalParametersAdvancedOptionsDialog.__init__(self)
        self.setupUi(self)
        title = self.tr("Advanced options for Global Numerical Parameters")
        self.setWindowTitle(title)
        self.default = default
        self.result  = self.default.copy()
        self.case    = case

        self.case.undoStopGlobal()

        # Combo model
        self.modelVelocityUpdate = ComboModel(self.comboBoxVelocityUpdate, 4, 1)
        self.modelVelocityUpdate.addItem(self.tr("By the pressure gradient increment"), "pressure_gradient_increment")
        self.modelVelocityUpdate.addItem(self.tr("RT0 by flow rate (internal+boundary)"), "flow_rate")
        self.modelVelocityUpdate.addItem(self.tr("RT0 by flumas(b) increment"), "flumas_increment")
        self.modelVelocityUpdate.addItem(self.tr("By means of Conv+Diff+ST equation"), "conv_diff_equation")

        self.modelGradientPressure = ComboModel(self.comboBoxGradientPressure, 5, 1)
        self.modelGradientPressure.addItem(self.tr("Mass ponderation"), "mass_ponderation")
        self.modelGradientPressure.addItem(self.tr("Standard"), "standard")
        self.modelGradientPressure.addItem(self.tr("Controlled mass ponderation"), "controlled_mass_pound")
        self.modelGradientPressure.addItem(self.tr("Momentum ponderation"), "momentum_pound")
        self.modelGradientPressure.addItem(self.tr("Gravity+HL+ST momentum"), "gravity_momentum")

        # Validator
        validatorSumAlpha = DoubleValidator(self.lineEditMaxSumAlpha, min = 0., max = 1.)
        validatorAlphaP   = IntValidator(self.lineEditNumberAlphaPCycle, min = 1)

        validatorSumAlpha.setExclusiveMin(False)
        validatorAlphaP.setExclusiveMin(False)

        self.lineEditMaxSumAlpha.setValidator(validatorSumAlpha)
        self.lineEditNumberAlphaPCycle.setValidator(validatorAlphaP)

        # Initialization
        if self.result['pressure_symetrisation'] == 'on' :
            self.checkBoxSymetPressure.setChecked(True)
        else :
            self.checkBoxSymetPressure.setChecked(False)

        self.modelVelocityUpdate.setItem(str_model=self.result['velocity_update'])
        self.modelGradientPressure.setItem(str_model=self.result['pressure_gradient'])
        self.lineEditMaxSumAlpha.setText(str(self.result['max_sum_alpha']))
        self.lineEditNumberAlphaPCycle.setText(str(self.result['alpha_p_cycle']))

        self.case.undoStartGlobal()


    def get_result(self):
        """
        Method to get the result
        """
        return self.result


    def accept(self):
        """
        Method called when user clicks 'OK'
        """
        if self.checkBoxSymetPressure.isChecked():
            self.result['pressure_symetrisation'] = 'on'
        else :
            self.result['pressure_symetrisation'] = 'off'

        self.result['velocity_update'] = \
                 self.modelVelocityUpdate.dicoV2M[str(self.comboBoxVelocityUpdate.currentText())]

        self.result['pressure_gradient'] = \
                 self.modelGradientPressure.dicoV2M[str(self.comboBoxGradientPressure.currentText())]

        self.result['max_sum_alpha']     = float(str(self.lineEditMaxSumAlpha.text()))
        self.result['alpha_p_cycle']     = int(str(self.lineEditNumberAlphaPCycle.text()))

        QDialog.accept(self)


    def reject(self):
        """
        Method called when user clicks 'Cancel'
        """
        QDialog.reject(self)


#-------------------------------------------------------------------------------
# GlobalNumericalParametersView class
#-------------------------------------------------------------------------------

class GlobalNumericalParametersView(QWidget, Ui_GlobalNumericalParameters):
    """
    Global numerical parameters layout.
    """
    def __init__(self, parent, case):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_GlobalNumericalParameters.__init__(self)
        self.setupUi(self)

        self.case = case
        self.case.undoStopGlobal()
        self.mdl = GlobalNumericalParametersModel(self.case)

        # Combo model
        self.modelVelocityAlgorithm = ComboModel(self.comboBoxVelocityAlgorithm, 3, 1)
        self.modelVelocityAlgorithm.addItem(self.tr("Standard"), "standard_difvit")
        self.modelVelocityAlgorithm.addItem(self.tr("Coupled"), "coupled_difvitc")
        self.modelVelocityAlgorithm.addItem(self.tr("Mean velocity - relative velocity"), "mean_velocity_relative_velocity")

        mfm = MainFieldsModel(self.case)
        if len(mfm.getFieldIdList()) < 2 :
            self.modelVelocityAlgorithm.disableItem(2)
        else :
            self.modelVelocityAlgorithm.enableItem(2)

        # Validator
        validatorMaxRestart = IntValidator(self.lineEditMaxRestart, min = 0)
        validatorSplitting  = DoubleValidator(self.lineEditTimeSplitting, min = 0.)
        validatorPRelax     = DoubleValidator(self.lineEditPressureRelaxation, min = 0.)
        validatorMinP       = DoubleValidator(self.lineEditMinimumPressure)
        validatorMaxP       = DoubleValidator(self.lineEditMaximumPressure)

        validatorMaxRestart.setExclusiveMin(False)
        validatorSplitting.setExclusiveMin(True)
        validatorPRelax.setExclusiveMin(True)

        self.lineEditMaxRestart.setValidator(validatorMaxRestart)
        self.lineEditTimeSplitting.setValidator(validatorSplitting)
        self.lineEditPressureRelaxation.setValidator(validatorPRelax)
        self.lineEditMinimumPressure.setValidator(validatorMinP)
        self.lineEditMaximumPressure.setValidator(validatorMaxP)

        # Connections
        self.checkBoxRestart.clicked.connect(self.slotRestart)
        self.checkBoxPotentialState.clicked.connect(self.slotPotentialState)
        self.checkBoxFacesReconstruction.clicked.connect(self.slotFacesReconstruction)
        self.checkBoxMultigrid.clicked.connect(self.slotMultigrid)
        self.lineEditMinimumPressure.textChanged[str].connect(self.slotMinimumPressure)
        self.lineEditMaximumPressure.textChanged[str].connect(self.slotMaximumPressure)
        self.pushButtonAdvanced.clicked.connect(self.slotAdvancedOptions)
        self.lineEditMaxRestart.textChanged[str].connect(self.slotMaxRestart)
        self.lineEditTimeSplitting.textChanged[str].connect(self.slotTimeSplitting)
        self.lineEditPressureRelaxation.textChanged[str].connect(self.slotPressureRelaxation)
        self.checkBoxUpwindAlphaEnergy.clicked.connect(self.slotUpwindAlphaEnergy)
        self.checkBoxStopRestart.clicked.connect(self.slotStopRestart)
        self.comboBoxVelocityAlgorithm.activated[str].connect(self.slotVelocityAlgorithm)

        self.checkBoxRegulBadCells.clicked.connect(self.slotRegulateBadCells)

        # Initialize widget
        status = self.mdl.getRestartTimeStep()
        if status == 'on':
            self.checkBoxRestart.setChecked(1)
            self.groupBoxRestartOption.show()

            value = self.mdl.getMaxNumberOfRestart()
            self.lineEditMaxRestart.setText(str(value))
            value = self.mdl.getTimeSplit()
            self.lineEditTimeSplitting.setText(str(value))
            value = self.mdl.getPressureRelaxation()
            self.lineEditPressureRelaxation.setText(str(value))
            status = self.mdl.getUpwindScheme()
            if status == 'on':
                self.checkBoxUpwindAlphaEnergy.setChecked(True)
            else :
                self.checkBoxUpwindAlphaEnergy.setChecked(False)
            status = self.mdl.getStopNoConvergence()
            if status == 'on':
                self.checkBoxStopRestart.setChecked(True)
            else :
                self.checkBoxStopRestart.setChecked(False)
        else :
            self.checkBoxRestart.setChecked(0)
            self.groupBoxRestartOption.hide()

        is_compressible = False
        for fid in mfm.getFieldIdList():
            if mfm.getCompressibleStatus(int(fid)) == 'on':
                is_compressible = True
                break

        if is_compressible:
            self.mdl.setPotentielState('off')
            self.checkBoxPotentialState.setChecked(0)
            self.checkBoxPotentialState.setEnabled(False)
        else:
            status = self.mdl.getPotentielState()
            if status == 'on':
                self.checkBoxPotentialState.setChecked(1)
            else :
                self.checkBoxPotentialState.setChecked(0)

        status = self.mdl.getFacesReconstruction()
        if status == 'on':
            self.checkBoxFacesReconstruction.setChecked(1)
        else :
            self.checkBoxFacesReconstruction.setChecked(0)

        status = self.mdl.getRegulateBadCElls()
        self.checkBoxRegulBadCells.setChecked(status == 'on')

        status = self.mdl.getMultigridStatus()
        if status == 'on':
            self.checkBoxMultigrid.setChecked(1)
        else :
            self.checkBoxMultigrid.setChecked(0)

        value = self.mdl.getMinPressure()
        self.lineEditMinimumPressure.setText(str(value))
        value = self.mdl.getMaxPressure()
        self.lineEditMaximumPressure.setText(str(value))

        model = self.mdl.getVelocityPredictorAlgo()
        self.modelVelocityAlgorithm.setItem(str_model=model)

        self.case.undoStartGlobal()

        predefined_flow = MainFieldsModel(self.case).getPredefinedFlow()
        if predefined_flow in ["free_surface", "droplet_flow", "multiregime"]:
            self.modelVelocityAlgorithm.setItem(str_model="coupled_difvitc")
            self.comboBoxVelocityAlgorithm.setEnabled(False)
        elif predefined_flow == "boiling_flow":
            self.modelVelocityAlgorithm.setItem(str_model="mean_velocity_relative_velocity")
            self.comboBoxVelocityAlgorithm.setEnabled(False)

    @pyqtSlot()
    def slotRestart(self):
        """
        Input if restart time step if not converged
        """
        if self.checkBoxRestart.isChecked():
            self.mdl.setRestartTimeStep('on')
            self.groupBoxRestartOption.show()

            value = self.mdl.getMaxNumberOfRestart()
            self.lineEditMaxRestart.setText(str(value))
            value = self.mdl.getTimeSplit()
            self.lineEditTimeSplitting.setText(str(value))
            value = self.mdl.getPressureRelaxation()
            self.lineEditPressureRelaxation.setText(str(value))
            status = self.mdl.getUpwindScheme()
            if status == 'on':
                self.checkBoxUpwindAlphaEnergy.setChecked(True)
            else :
                self.checkBoxUpwindAlphaEnergy.setChecked(False)
            status = self.mdl.getStopNoConvergence()
            if status == 'on':
                self.checkBoxStopRestart.setChecked(True)
            else :
                self.checkBoxStopRestart.setChecked(False)
        else:
            self.mdl.setRestartTimeStep('off')
            self.groupBoxRestartOption.hide()


    @pyqtSlot()
    def slotPotentialState(self):
        """
        Input if restart time step if not converged
        """
        if self.checkBoxPotentialState.isChecked():
            self.mdl.setPotentielState('on')
        else:
            self.mdl.setPotentielState('off')


    @pyqtSlot()
    def slotFacesReconstruction(self):
        """
        Input if faces reconstruction
        """
        if self.checkBoxFacesReconstruction.isChecked():
            self.mdl.setFacesReconstruction('on')
            self.checkBoxFacesReconstruction.setChecked(1)
        else:
            self.mdl.setFacesReconstruction('off')


    @pyqtSlot()
    def slotMultigrid(self):
        """
        Input if multigrid for pressure
        """
        if self.checkBoxMultigrid.isChecked():
            self.mdl.setMultigridStatus('on')
        else:
            self.mdl.setMultigridStatus('off')


    @pyqtSlot(str)
    def slotMinimumPressure(self, text):
        """
        Input value of minimum pressure
        """
        if self.lineEditMinimumPressure.validator().state == QValidator.Acceptable:
            value = from_qvariant(text, float)
            self.mdl.setMinPressure(value)


    @pyqtSlot(str)
    def slotMaximumPressure(self, text):
        """
        Input value of maximum pressure
        """
        if self.lineEditMaximumPressure.validator().state == QValidator.Acceptable:
            value = from_qvariant(text, float)
            self.mdl.setMaxPressure(value)


    @pyqtSlot()
    def slotAdvancedOptions(self):
        """
        Ask one popup for advanced specifications
        """
        default = {}
        default['velocity_update'] = self.mdl.getVelocityUpdate()
        default['pressure_symetrisation'] = self.mdl.getPressureSymetrisation()
        default['pressure_gradient'] = self.mdl.getPressureGradient()
        default['max_sum_alpha'] = self.mdl.getSumAlpha()
        default['alpha_p_cycle'] = self.mdl.getAlphaPressureCycles()
        log.debug("slotAdvancedOptions -> %s" % str(default))

        dialog = GlobalNumericalParametersAdvancedOptionsDialogView(self, self.case, default)
        if dialog.exec_():
            result = dialog.get_result()
            log.debug("slotAdvancedOptions -> %s" % str(result))
            self.mdl.setVelocityUpdate(result['velocity_update'])
            self.mdl.setPressureSymetrisation(result['pressure_symetrisation'])
            self.mdl.setPressureGradient(result['pressure_gradient'])
            self.mdl.setSumAlpha(result['max_sum_alpha'])
            self.mdl.setAlphaPressureCycles(result['alpha_p_cycle'])


    @pyqtSlot(str)
    def slotMaxRestart(self, text):
        """
        Input value of Maximum number of restart
        """
        if self.lineEditMaxRestart.validator().state == QValidator.Acceptable:
            value = from_qvariant(text, int)
            self.mdl.setMaxNumberOfRestart(value)


    @pyqtSlot(str)
    def slotTimeSplitting(self, text):
        """
        Input value of time-step splitting
        """
        if self.lineEditTimeSplitting.validator().state == QValidator.Acceptable:
            value = from_qvariant(text, float)
            self.mdl.setTimeSplit(value)


    @pyqtSlot(str)
    def slotPressureRelaxation(self, text):
        """
        Input value of pressure increment relaxation
        """
        if self.lineEditPressureRelaxation.validator().state == QValidator.Acceptable:
            value = from_qvariant(text, float)
            self.mdl.setPressureRelaxation(value)


    @pyqtSlot()
    def slotUpwindAlphaEnergy(self):
        """
        Input if upwind scheme for mass and energy
        """
        if self.checkBoxUpwindAlphaEnergy.isChecked():
            self.mdl.setUpwindScheme('on')
        else:
            self.mdl.setUpwindScheme('off')


    @pyqtSlot()
    def slotStopRestart(self):
        """
        Input if stop if no convergence
        """
        if self.checkBoxStopRestart.isChecked():
            self.mdl.setStopNoConvergence('on')
        else:
            self.mdl.setStopNoConvergence('off')


    @pyqtSlot(str)
    def slotVelocityAlgorithm(self, text):
        """
        Input velocity algorithm model
        """
        model = self.modelVelocityAlgorithm.dicoV2M[str(text)]
        self.mdl.setVelocityPredictorAlgo(model)


    @pyqtSlot()
    def slotRegulateBadCells(self):
        """
        Activate bad cells regulations.
        """
        if self.checkBoxRegulBadCells.isChecked():
            self.mdl.setRegulateBadCells('on')
        else:
            self.mdl.setRegulateBadCells('off')




# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

import sys, unittest
from code_saturne.model.XMLvariables import Model
from code_saturne.model.XMLengine import *
from code_saturne.model.XMLmodel import *
from code_saturne.model.MainFieldsModel import MainFieldsModel
from code_saturne.model.TimeAveragesModel import TimeAveragesModel

#-------------------------------------------------------------------------------
# Constructor
#-------------------------------------------------------------------------------

class OutputFieldsModel(MainFieldsModel, Variables, Model):
    """
    This class manages the Field objects in the XML file
    """

    def __init__(self, case):
        """
        Constuctor.
        """
        #
        # XML file parameters
        MainFieldsModel.__init__(self, case)
        self.case           = case
        self.XMLNodethermo  = self.case.xmlGetNode('thermophysical_models')
        self.analysis_ctrl  = self.case.xmlInitNode('analysis_control')
        self.node_output    = self.analysis_ctrl.xmlInitNode('output')
        self.node_probe     = self.node_output.xmlGetNodeList('probe','name')


    def defaultValues(self):
        default = {}
        default['listing'] = 'on'
        default['writer']  = 'on'
        return default


    def getVariableLabelsList(self) :
        """
        Return list of label for variable
        """
        lst = []
        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            for node in self.case.xmlGetNodeList(variableType) :
                lst.append(node['label'])
        return lst


    def getVariableProbeList(self):
        """
        Return list of node for probes
        Only for the View
        """
        probeList = []
        for node in self.node_probe:
            probeList.append(node['name'])
        return probeList


    @Variables.undoLocal
    def setVariableLabel(self, fieldId, label, oldlabel) :
        """
        set label
        """
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), label = oldlabel)
            if node != None:
                break

        if node != None:
            if variableType != 'time_average':
                node['label'] = label
            else:
                TimeAveragesModel(self.case).setLabel(node['label'], label)

        else :
            msg = "This variable " + oldlabel + " doesn't exist"
            raise ValueError(msg)

        # udpate label in formula if needed
        if oldlabel != '':
            for no in self.case.xmlGetNodeList('formula'):
                txt = no.xmlGetTextNode()
                if txt != None:
                    f = txt.replace(oldlabel, label)
                    no.xmlSetTextNode(f)


    @Variables.noUndo
    def getVariableLabel(self, fieldId, name) :
        """
        return label of name variable for fieldId
        """
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), name = name)
            if node != None:
                break

        if node != None:
            label = node['label']
            return label
        else :
            msg = "This variable " + name + " doesn't exist"
            raise ValueError(msg)


    @Variables.undoLocal
    def setListingStatus(self, fieldId, label, status) :
        """
        return status for listing output for variable name on fieldId
        """
        self.isOnOff(status)
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), label = label)
            if node != None:
                break

        if node != None:
            n = node.xmlGetNode('listing_printing')
            n['status'] = status
        else :
            msg = "This variable " + label + " doesn't exist"
            raise ValueError(msg)


    @Variables.noUndo
    def getListingStatus(self, fieldId, name) :
        """
        return status for listing output for variable name on fieldId
        """
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), name = name)
            if node != None:
                break

        if node != None:
            value = self.defaultValues()['listing']
            n = node.xmlGetNode('listing_printing')
            if n :
                value = n['status']
            return value
        else :
            msg = "This variable " + name + " doesn't exist"
            raise ValueError(msg)


    @Variables.undoLocal
    def setPostProcessingStatus(self, fieldId, label, status) :
        """
        return status for post processing for variable name on fieldId
        """
        self.isOnOff(status)
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), label = label)
            if node != None:
                break

        if node != None:
            n = node.xmlGetNode('postprocessing_recording')
            n['status'] = status
        else :
            msg = "This variable " + label + " doesn't exist"
            raise ValueError(msg)


    @Variables.noUndo
    def getPostProcessingStatus(self, fieldId, name) :
        """
        return status for post processing for variable name on fieldId
        """
        lst = self.getFieldIdList()
        lst.append("none")
        self.isInList(fieldId, lst)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), name = name)
            if node != None:
                break

        if node != None:
            value = self.defaultValues()['writer']
            n = node.xmlGetNode('postprocessing_recording')
            if n :
                value = n['status']
            return value
        else :
            msg = "This variable " + name + " doesn't exist"
            raise ValueError(msg)


    @Variables.undoGlobal
    def setProbesList(self, fieldId, label, probes):
        """
        return list of probes for variable name on fieldId
        """
        l = self.getFieldIdList()
        l.append("none")
        self.isInList(fieldId, l)

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), label = label)
            if node != None:
                break
        if not node:
            msg = "This variable " + label + " doesn't exist"
            raise ValueError(msg)

        l1 = string.split(probes)
        l2 = self.getVariableProbeList()
        l1.sort()
        l2.sort()

        if l1 == l2:
            # if all probes are selected: no markup is nedded in the xml file
            node.xmlRemoveChild('probes')
            node.xmlRemoveChild('no_probe')
            return
        else:
            try:
                node.xmlRemoveChild('probes')
                node.xmlRemoveChild('no_probe')
            except:
                pass

            if l1:
                n = node.xmlInitNode('probes')
                l3 = []
                for probe in l1:
                    if probe not in l3 and probe in l2:
                        l3.append(probe)
                        n.xmlInitChildNodeList('probe_recording', name = probe)
            else:
                n = node.xmlInitNode('no_probe')


    @Variables.noUndo
    def getProbesList(self, fieldId, name):
        """
        return list of probes for variable name on fieldId
        """
        l = self.getFieldIdList()
        l.append("none")
        self.isInList(fieldId, l)

        l1 = self.getVariableProbeList()

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            node = self.case.xmlGetNode(variableType, field_id = str(fieldId), name = name)
            if node != None:
                break
        else:
            msg = "This variable " + name + " doesn't exist"
            raise ValueError(msg)


        node_probes = node.xmlGetChildNode('probes')

        if node.xmlGetChildNode('no_probe'):
            if node_probes:
                node_probes.xmlRemoveNode()
            return []

        if node_probes:
            l2 = node_probes.xmlGetChildNodeList('probe_recording')
            l3 = []

            if l2 == []:
                node_probes.xmlRemoveNode()
            else:
                for n in l2:
                    if n['name'] not in l3:
                        l3.append(n['name'])

            return l3
        else:
            return l1


    def getGlobalVariables(self) :
        """
        return list of variables with none field criteria
        """
        llst = {}
        lst = []

        for variableType in ('variable', 'property', 'scalar', 'time_average') :
            for node in self.case.xmlGetNodeList(variableType, field_id = "none"):
                if not node['name'].startswith("User_"):
                    lst.append(node['name'])
                else:
                    idx = node['name'].split('_')[1]
                    tmp = idx.zfill(4)
                    llst[tmp] = node['name']
        keys = list(llst.keys())
        keys.sort()

        for key in keys:
            lst.append(llst[key])

        return lst


    def getFieldVariables(self, fieldId) :
        """
        return list of variables with none field criteria
        """
        self.isInList(str(fieldId),self.getFieldIdList())
        lst = []
        for variableType in ('variable', 'property', 'scalar'):
            for node in self.case.xmlGetNodeList(variableType, field_id = str(fieldId)):
                lst.append(node['name'])

        return lst

#-------------------------------------------------------------------------------
# DefineUsersScalars test case
#-------------------------------------------------------------------------------

class OutputFieldsTestCase(ModelTest):
    """
    """
    def checkOutputFieldsInstantiation(self):
        """Check whether the OutputFieldsModel class could be instantiated"""
        model = None
        model = OutputFieldsModel(self.case)
        assert model != None, 'Could not instantiate OutputFieldsModel'


    def checkGetVariableLabelsList(self):
        """Check whether the  OutputFieldsModel class could get the VariableLabelsList"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        assert mdl.getVariableLabelsList() == ['Pressure', 'enthalpy1', 'alpha1', 'U1', 'V1', 'W1', 'Temp1', 'density1', 'Lam_vis1', 'Sp_heat1', 'Th_cond1', 'mass_trans1'],\
            'Could not get VariableLabelsList'


    def checkGetandSetVariableLabel(self):
        """Check whether the OutputFieldsModel class could set and get VariableLabel"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        mdl.setVariableLabel('1','Vitesse1','U1')
        doc = '''<variable field_id="1" label="Vitesse1" name="velocityX">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                 </variable>'''
        assert mdl.case.xmlGetNode('variable', field_id = str('1'), label = 'Vitesse1') == self.xmlNodeFromString(doc),\
            'Could not set VariableLabel'
        assert mdl.getVariableLabel('1','velocityX') == 'Vitesse1',\
            'Could not get VariableLabel'


    def checkGetandSetListingStatus(self):
        """Check whether the OutputFieldsModel class could set and get ListingStatus"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        mdl.setListingStatus('1','U1','off')
        doc = '''<variable field_id="1" label="U1" name="velocityX">
                         <listing_printing status="off"/>
                         <postprocessing_recording status="on"/>
                 </variable>'''
        assert mdl.case.xmlGetNode('variable', field_id = str('1'), label = 'U1') == self.xmlNodeFromString(doc),\
            'Could not set ListingStatus'
        assert mdl.getListingStatus('1','velocityX') == 'off',\
            'Could not get ListingStatus'


    def checkGetandSetPostProcessingStatus(self):
        """Check whether the OutputFieldsModel class could set and get PostProcessingStatus"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        mdl.setPostProcessingStatus('1','U1','off')
        doc = '''<variable field_id="1" label="U1" name="velocityX">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="off"/>
                 </variable>'''
        assert mdl.case.xmlGetNode('variable', field_id = str('1'), label = 'U1') == self.xmlNodeFromString(doc),\
            'Could not set PostProcessingStatus'
        assert mdl.getPostProcessingStatus('1','velocityX') == 'off',\
            'Could not get PostProcessingStatus'


    def checkGetandSetProbesList(self):
        """Check whether the OutputFieldsModel class could set and get ProbesList"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        mdl.setProbesList('1','U1','1')
        doc = '''<variable field_id="1" label="U1" name="velocityX">
                         <listing_printing status="on"/>
                         <postprocessing_recording status="on"/>
                         <probes>
                                 <probe_recording name="1"/>
                         </probes>
                 </variable>'''
        assert mdl.case.xmlGetNode('variable', field_id = str('1'), label = 'U1') == self.xmlNodeFromString(doc),\
            'Could not set ProbesList'
        assert mdl.getProbesList('1','velocityX') == ['1'],\
            'Could not get ProbesList'


    def checkGetGlobalVariables(self):
        """Check whether the  OutputFieldsModel class could get GlobalVariables"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        assert mdl.getGlobalVariables() == ['Pressure'],\
            'Could not get GlobalVariables'


    def checkGetFieldVariables(self):
        """Check whether the  OutputFieldsModel class could get FieldVariables"""
        MainFieldsModel(self.case).addField()
        mdl = OutputFieldsModel(self.case)
        assert mdl.getFieldVariables('1') == ['Enthalpy', 'VolumeFraction', 'VelocityX', 'VelocityY', 'VelocityZ', 'Temperature', 'density', 'molecular_viscosity', 'specific_heat', 'thermal_conductivity', 'mass_trans'],\
            'Could not get FieldVariables'


def suite():
    testSuite = unittest.makeSuite(OutputFieldsTestCase, "check")
    return testSuite


def runTest():
    print("OutputFieldsTestCase")
    runner = unittest.TextTestRunner()
    runner.run(suite())



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the conjugate heat transfer model data management.

This module contains the following classes and function:
- ConjugateHeatTransferModel
- ConjugateHeatTransferModelTestCase
"""

#-------------------------------------------------------------------------------
# Library modules
#-------------------------------------------------------------------------------

import sys, unittest

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import *
from code_saturne.model.XMLvariables import Variables, Model
from code_saturne.model.XMLmodel import ModelTest

#-------------------------------------------------------------------------------
# Conjugate Heat Transfer model class
#-------------------------------------------------------------------------------

class ConjugateHeatTransferModel(Variables, Model):
    """
    Manage the input/output markups in the xml doc about HeadLosses
    """
    def __init__(self, case):
        """
        Constructor.
        """
        self.case = case

        self.__node_models = self.case.xmlGetNode('thermophysical_models')
        self.__node_cht    = self.__node_models.xmlInitNode('conjugate_heat_transfer')
        self.__node_syr = self.__node_cht.xmlInitNode('external_coupling')
        self.__node_inst = self.__node_syr.xmlInitNode('syrthes_instances')

    @Variables.undoLocal
    def deleteConjugateHeatTransfer(self):
        """
        Update the 'Conjugate heat transfer' status.
        @type status: C{String}
        @param status: set to "on" or "off" the conjugate heat transfer
        """
        self.__node_inst.xmlRemoveNode()

    def getNumberOfSyrthesCoupling(self):
        return len(self.__node_inst.xmlGetNodeList('instance'))

    @Variables.noUndo
    def getSyrthesInstancesList(self):
        """
        @return: list of syrthes instances
        """

        lst = []
        node_list = self.__node_inst.xmlGetNodeList('instance')

        for node in node_list:
            syrthes_name = node.xmlGetAttribute("name")
            if syrthes_name and syrthes_name not in lst:
                lst.append(syrthes_name)

        return lst

    @Variables.noUndo
    def getSyrthesCouplingList(self):
        """
        @return: list of Syrthes coupling description.
        @rtype: C{List}
        """
        node_list = self.__node_inst.xmlGetNodeList('instance')
        lst = []
        for node in node_list:
            syrthes_name = node.xmlGetAttribute("name")
            boundary_labels = self.getBoundaryLabelList(syrthes_name)
            lst.append([syrthes_name, boundary_labels])
        return lst

    @Variables.undoGlobal
    def addSyrthesCoupling(self, syrthes_name):
        """
        Add a new definition of a Syrthes coupling.

        @type syrthes_name: C{String}
        @param syrthes_name: Syrthes instance name
        """
        node_new = self.__node_inst.xmlInitChildNode('instance', name=syrthes_name)
        return


    @Variables.undoGlobal
    def deleteSyrthesCoupling(self, syrthes_name, boundary_label):
        """
        Delete a syrthes coupling. If the syrthes instance has no
        other coupled boundaries, remove it from the instances list.
        """

        node = self.__node_inst.xmlGetChildNode('instance', name=syrthes_name)
        if node:
            # First delete coupling
            for node_b in node.xmlGetChildNodeList("coupled_boundary"):
                if node_b.xmlGetAttribute("label") == boundary_label:
                    node_b.xmlRemoveNode()
                    break

            # If instance has no other coupling, delete it
            if not node.xmlGetChildNodeList("coupled_boundary"):
                node.xmlRemoveNode()



    def addBoundaryLabel(self, syrthes_name, boundary_label):

        # If boundary allready added, exit
        if boundary_label in self.getBoundaryLabelList(syrthes_name):
            return

        node_instance = self.__node_inst.xmlGetChildNode('instance', name=syrthes_name)
        if node_instance == None:
            raise ValueError("SYRTHES instance not found : ", syrthes_name)
        node_zone = node_instance.xmlInitChildNode('coupled_boundary', label=boundary_label)

    def getBoundaryLabelList(self, syrthes_name):
        node_instance = self.__node_inst.xmlGetChildNode('instance', name=syrthes_name)
        if node_instance == None:
            raise ValueError("SYRTHES instance not found : ", syrthes_name)
        return [n.xmlGetAttribute("label") for n in node_instance.xmlGetChildNodeList("coupled_boundary")]

    def flushBoundaryLabels(self):
        for node_instance in self.__node_inst.xmlGetChildNodeList('instance'):
            node_instance.xmlRemoveChildren()

    @Variables.undoLocal
    def setSyrthesVerbosity(self, value):
        self.__node_syr.xmlSetData('verbosity', value)

    @Variables.noUndo
    def getSyrthesVerbosity(self):
        return self.__node_syr.xmlGetChildString('verbosity')

    @Variables.undoLocal
    def setSyrthesVisualization(self, value):
        self.__node_syr.xmlSetData('visualization', value)

    @Variables.noUndo
    def getSyrthesVisualization(self):
        return self.__node_syr.xmlGetChildString('visualization')

    @Variables.undoLocal
    def setSyrthesProjectionAxis(self, value):
        self.__node_syr.xmlSetData('projection_axis', value)

    @Variables.noUndo
    def getSyrthesProjectionAxis(self):
        return self.__node_syr.xmlGetChildString('projection_axis')

    @Variables.undoLocal
    def setSyrthesTolerance(self, value):
        self.__node_syr.xmlSetData('tolerance', value)

    @Variables.noUndo
    def getSyrthesTolerance(self):

        val = self.__node_syr.xmlGetChildString('tolerance')
        if not val or val == "":
            val = "0.1"
            self.setSyrthesTolerance(val)

        return val

#-------------------------------------------------------------------------------
# ConjugateHeatTransferModel test case
#-------------------------------------------------------------------------------


class ConjugateHeatTransferModelTestCase(ModelTest):
    """
    """
    def checkConjugateHeatTransferInstantiation(self):
        """Check whether the ConjugateHeatTransfer class could be instantiated"""
        model = None
        model = ConjugateHeatTransferModel(self.case)
        assert model != None, 'Could not instantiate HeadLossesModel'


def suite():
    testSuite = unittest.makeSuite(ConjugateHeatTransferModelTestCase, "check")
    return testSuite

def runTest():
    print("ConjugateHeatTransferModelTestCase")
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the 'Interfacial area' page.

This module contains the following classes:
- InterfacialAreaView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import os, sys, string, types
import logging

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import GuiParam
from code_saturne.Base.QtPage import ComboModel, DoubleValidator, from_qvariant
from code_saturne.Pages.InterfacialArea import Ui_InterfacialArea
from code_saturne.model.InterfacialAreaModel import InterfacialAreaModel
from code_saturne.model.MainFieldsModel import MainFieldsModel
from code_saturne.model.InterfacialForcesModel import InterfacialForcesModel
#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("InterfacialAreaView")
log.setLevel(GuiParam.DEBUG)


#-------------------------------------------------------------------------------
# SolidView class
#-------------------------------------------------------------------------------

class InterfacialAreaView(QWidget, Ui_InterfacialArea):
    """
    InterfacialAreaView layout.
    """
    def __init__(self, parent, case):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_InterfacialArea.__init__(self)
        self.setupUi(self)

        self.case = case
        self.case.undoStopGlobal()
        self.mdl = InterfacialAreaModel(self.case)

        dispersed_fields = self.mdl.getDispersedFieldList() + InterfacialForcesModel(self.case).getGLIMfields()

        if dispersed_fields == []:
            self.groupBoxField.hide()
            self.groupBoxMinMaxDiameter.hide()
            self.groupBoxModel.hide()
            self.labelNoDispersedPhase.show()
            self.mdl.remove()
            return

        # Combo box models
        id_to_set = -1
        self.modelField = ComboModel(self.comboBoxField, 1, 1)

        # For consistency with the previous pages, the second phase of the
        # Large Interface Model is set before the dispersed fields

        for fieldId in dispersed_fields:
            label = self.mdl.getLabel(fieldId)
            name = str(fieldId)
            self.modelField.addItem(self.tr(label), name)

        if len(dispersed_fields) > 0 and id_to_set == -1:
            id_to_set = dispersed_fields[0]
            self.modelField.setItem(str_model=id_to_set)

        # case no field
        self.currentid = id_to_set

        self.modelModel = ComboModel(self.comboBoxModel, 2, 1)
        self.modelModel.addItem(self.tr("constant"), "constant")
        self.modelModel.addItem(self.tr("interfacial area transport"), "interfacial_area_transport")

        self.modelSourceTerm = ComboModel(self.comboBoxSourceTerm, 4, 1)

        self.modelSourceTerm.addItem(self.tr("No coalescence, no fragmentation"),"no_coalescence_no_fragmentation")
        self.modelSourceTerm.addItem(self.tr("Yao & Morel"),"wei_yao")
        self.modelSourceTerm.addItem(self.tr("Kamp & Colin"),"kamp_colin")
        self.modelSourceTerm.addItem(self.tr("Ruyer & Seiler"), "ruyer_seiler")
        self.modelSourceTerm.disableItem(2)  # Why ?

        # Validators
        validatorDefDiam = DoubleValidator(self.lineEditDefaultDiameter, min = 0.0)
        validatorMinDiam = DoubleValidator(self.lineEditMinDiameter, min = 0.0)
        validatorMaxDiam = DoubleValidator(self.lineEditMaxDiameter, min = 0.0)

        validatorDefDiam.setExclusiveMin(True)
        validatorMinDiam.setExclusiveMin(True)
        validatorMaxDiam.setExclusiveMin(True)

        self.lineEditDefaultDiameter.setValidator(validatorDefDiam)
        self.lineEditMinDiameter.setValidator(validatorMinDiam)
        self.lineEditMaxDiameter.setValidator(validatorMaxDiam)

        # Connect signals to slots
        self.comboBoxField.activated[str].connect(self.slotField)
        self.comboBoxModel.activated[str].connect(self.slotModel)
        self.comboBoxSourceTerm.activated[str].connect(self.slotSourceTerm)
        self.lineEditDefaultDiameter.textChanged[str].connect(self.slotDefaultDiameter)
        self.lineEditMinDiameter.textChanged[str].connect(self.slotMinDiameter)
        self.lineEditMaxDiameter.textChanged[str].connect(self.slotMaxDiameter)

        # Initialize widget
        self.initializeVariables(self.currentid)

        self.case.undoStartGlobal()


    @pyqtSlot(str)
    def slotField(self, text):
        """
        INPUT label for choice of field
        """
        self.currentid = self.modelField.dicoV2M[text]
        self.initializeVariables(self.currentid)

        if self.mdl.getFieldNature(self.currentid) == "gas" :
            self.modelSourceTerm.enableItem(0)
        else :
            self.modelSourceTerm.disableItem(0)


    @pyqtSlot(str)
    def slotModel(self, text):
        """
        INPUT type for choice of model
        """
        model = self.modelModel.dicoV2M[text]
        self.mdl.setAreaModel(self.currentid, model)
        self.initializeVariables(self.currentid)


    @pyqtSlot(str)
    def slotSourceTerm(self, text):
        """
        INPUT type for choice of model source term
        """
        model = self.modelSourceTerm.dicoV2M[text]
        self.mdl.setSourceTerm(self.currentid, model)


    @pyqtSlot(str)
    def slotDefaultDiameter(self, var):
        """
        """
        if self.lineEditDefaultDiameter.validator().state == QValidator.Acceptable:
            value = from_qvariant(var, float)
            self.mdl.setInitialDiameter(self.currentid, value)


    @pyqtSlot(str)
    def slotMinDiameter(self, var):
        """
        """
        if self.lineEditMinDiameter.validator().state == QValidator.Acceptable:
            value = from_qvariant(var, float)
            self.mdl.setMinDiameter(self.currentid, value)


    @pyqtSlot(str)
    def slotMaxDiameter(self, var):
        """
        """
        if self.lineEditMaxDiameter.validator().state == QValidator.Acceptable:
            value = from_qvariant(var, float)
            self.mdl.setMaxDiameter(self.currentid, value)


    def initializeVariables(self, fieldId):
        """
        Initialize variables when a new fieldId is choosen
        """
        self.labelNoDispersedPhase.hide()
        model = self.mdl.getAreaModel(fieldId)
        self.modelModel.setItem(str_model = model)

        value = self.mdl.getInitialDiameter(self.currentid)
        self.lineEditDefaultDiameter.setText(str(value))

        if self.mdl.getAreaModel(fieldId) == "constant" :
            self.groupBoxAreaTransport.hide()
            self.groupBoxMinMaxDiameter.hide()
        else :
            self.groupBoxAreaTransport.show()
            model = self.mdl.getSourceTerm(fieldId)
            self.modelSourceTerm.setItem(str_model = model)

            self.groupBoxMinMaxDiameter.show()

            value = self.mdl.getMinDiameter(self.currentid)
            self.lineEditMinDiameter.setText(str(value))

            value = self.mdl.getMaxDiameter(self.currentid)
            self.lineEditMaxDiameter.setText(str(value))

            if MainFieldsModel(self.case).getFieldNature(fieldId) != 'gas' :
                self.modelSourceTerm.disableItem(1)
                self.modelSourceTerm.disableItem(2)
                self.modelSourceTerm.disableItem(3)
            else:
                if MainFieldsModel(self.case).getPredefinedFlow() == "multiregime":
                    self.modelSourceTerm.disableItem(1)
                    self.modelSourceTerm.disableItem(2)
                    self.modelSourceTerm.enableItem(3)
                else:
                    self.modelSourceTerm.enableItem(1)
                    self.modelSourceTerm.enableItem(2)
                    self.modelSourceTerm.enableItem(3)



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module defines the XML calls for preprocessor execution
This module contains the following classes and function:
- BalanceModel
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import sys, unittest
import os, sys, types

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.XMLvariables import Variables
from code_saturne.model.XMLvariables import Model
from code_saturne.model.BalanceModel import *

#-------------------------------------------------------------------------------
# Class Balance Model
#-------------------------------------------------------------------------------

class BalanceModelNeptune(BalanceModel, Variables, Model):

    """
    This class manages the scalar balance objects in the XML file
    """

    def __init__(self, case):
        """
        Constuctor.
        """
        BalanceModel.__init__(self, case)


    @Variables.noUndo
    def __getVariables__(self):

        # Get fields:
        fd = []
        fd.append('none')
        thermo = self.case.xmlGetNode('thermophysical_models')
        fields = thermo.xmlGetNode('fields')
        for node in fields.xmlInitChildNodeList('field'):
            field_id = node.xmlGetAttribute('field_id')
            fd.append(field_id)

        l = []
        for variableType in ('variable', 'property', 'scalar'):
            for field in fd:
                for node in self.case.xmlGetNodeList(variableType, field_id = field):
                    l.append(node)

        return l


    @Variables.noUndo
    def getScalarVariables(self):
        """
        Creates a dictionnary to connect name and label of
        scalar variables (different pressure).
        """
        self.dicoLabel2Name = {}

        for nodeList in [self.getVariables()]:

            for node in nodeList:

                name = node['name']
                label = node['label']
                if not label:
                    raise ValueError("Node has no label")

                dim = node['dimension']
                if not dim or int(dim) == 1:
                    if name != "pressure":
                        self.dicoLabel2Name[label] = (name, str(0))

        return list(self.dicoLabel2Name.keys())



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module contains the following classes and function:
- TimeStepView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import logging

#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import GuiParam
from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator, from_qvariant
from code_saturne.Pages.TimeStepForm import Ui_TimeStepForm
from code_saturne.model.TimeStepModel import TimeStepModel

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("TimeStepView")
log.setLevel(GuiParam.DEBUG)

#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------

class TimeStepView(QWidget, Ui_TimeStepForm):
    """
    """
    def __init__(self, parent, case, tree):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_TimeStepForm.__init__(self)
        self.setupUi(self)

        self.case = case
        self.case.undoStopGlobal()
        self.mdl = TimeStepModel(self.case)
        self.browser = tree

        # Check if xml contains deprecated steady algorithme
        has_deprecated_steady = False
        if str(self.mdl.getTimePassing()) == '-1':
            has_deprecated_steady = True

       # Combo model
        if has_deprecated_steady:
            self.modelTimeOptions = ComboModel(self.comboBoxOptions,4,1)
        else:
            self.modelTimeOptions = ComboModel(self.comboBoxOptions,3,1)
        self.modelTimeOptions.addItem(self.tr("Constant"), '0')
        self.modelTimeOptions.addItem(self.tr("Time varying (adaptive)"), '1')
        self.modelTimeOptions.addItem(self.tr("Steady (local time step)"), '2')
        if has_deprecated_steady:
            self.modelTimeOptions.addItem(self.tr("Deprecated: steady (constant relaxation coefficient)"), '-1')

        self.modelNTERUP = ComboModel(self.comboBoxNTERUP,3,1)
        self.modelNTERUP.addItem(self.tr("SIMPLE"), 'simple')
        self.modelNTERUP.addItem(self.tr("SIMPLEC"), 'simplec')
        self.modelNTERUP.addItem(self.tr("Inner iterations"), 'piso')
        self.comboBoxNTERUP.setSizeAdjustPolicy(QComboBox.AdjustToContents)

        self.modelTimeStop = ComboModel(self.comboBoxStopCrit, 2, 1)
        self.modelTimeStop.addItem(self.tr("Number of time steps"), "iterations")
        self.modelTimeStop.addItem(self.tr("Physical time (s)"), "maximum_time")
        self.modelTimeStop.addItem(self.tr("Additional time steps"), "iterations_add")
        self.modelTimeStop.addItem(self.tr("Additional physical time (s)"), "maximum_time_add")

        # Connections
        self.comboBoxOptions.activated[str].connect(self.slotTimePassing)
        self.lineEditDTREF.textChanged[str].connect(self.slotTimeStep)
        self.lineEditRELXST.textChanged[str].connect(self.slotRelaxCoef)
        self.lineEditCOUMAX.textChanged[str].connect(self.slotTimeOptionCOUMAX)
        self.lineEditFOUMAX.textChanged[str].connect(self.slotTimeOptionFOUMAX)
        self.lineEditCDTMIN.textChanged[str].connect(self.slotTimeOptionCDTMIN)
        self.lineEditCDTMAX.textChanged[str].connect(self.slotTimeOptionCDTMAX)
        self.lineEditVARRDT.textChanged[str].connect(self.slotTimeOptionVARRDT)
        self.checkBoxIPTLRO.clicked.connect(self.slotThermalTimeStep)
        self.comboBoxNTERUP.activated[str].connect(self.slotNTERUP)
        self.spinBoxNTERUP.valueChanged[int].connect(self.slotNTERUP2)
        self.comboBoxStopCrit.activated[str].connect(self.slotStopCritModel)
        self.lineEditStop.textChanged[str].connect(self.slotStopCritValue)

        # Validators

        validatorDTREF = DoubleValidator(self.lineEditDTREF, min=0.0)
        validatorDTREF.setExclusiveMin(True)
        validatorRELXST = DoubleValidator(self.lineEditRELXST, min=0.0, max=1.0)
        validatorRELXST.setExclusiveMin(True)
        validatorCOUMAX = DoubleValidator(self.lineEditCOUMAX, min=0.0)
        validatorCOUMAX.setExclusiveMin(True)
        validatorFOUMAX = DoubleValidator(self.lineEditFOUMAX, min=0.0)
        validatorFOUMAX.setExclusiveMin(True)
        validatorCDTMIN = DoubleValidator(self.lineEditCDTMIN, min=0.0, max=1.0)
        validatorCDTMIN.setExclusiveMin(True)
        validatorCDTMAX = DoubleValidator(self.lineEditCDTMAX, min=1.0)
        validatorVARRDT = DoubleValidator(self.lineEditVARRDT, min=0.0)
        validatorVARRDT.setExclusiveMin(True)

        self.lineEditDTREF.setValidator(validatorDTREF)
        self.lineEditRELXST.setValidator(validatorRELXST)
        self.lineEditCOUMAX.setValidator(validatorCOUMAX)
        self.lineEditFOUMAX.setValidator(validatorFOUMAX)
        self.lineEditCDTMIN.setValidator(validatorCDTMIN)
        self.lineEditCDTMAX.setValidator(validatorCDTMAX)
        self.lineEditVARRDT.setValidator(validatorVARRDT)

        self.validatorNTABS = IntValidator(self.lineEditStop, min=0)
        self.validatorTABS = DoubleValidator(self.lineEditStop, min=0.0)

        # Initialization

        idtvar = self.mdl.getTimePassing()
        idtvar_p = idtvar

        # Constraints on time step from Turbulence model

        from code_saturne.model.TurbulenceModel import TurbulenceModel
        model = TurbulenceModel(self.case).getTurbulenceModel()
        del TurbulenceModel

        if model in ('LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
            idtvar = 0

            # Deactivate all modes except constant time step
            for itm in self.modelTimeOptions.getItems():
                if str(itm) != "0":
                    self.modelTimeOptions.disableItem(str_model=itm)

        # Constraints on time step from Lagrangian model

        from code_saturne.model.LagrangianModel import LagrangianModel
        model = LagrangianModel(self.case).getLagrangianModel()
        if model in ['one_way', 'two_way']:
            if idtvar not in [0, 1]:
                idtvar = 0

            # Deactivate certain modes
            for itm in self.modelTimeOptions.getItems():
                if str(itm) == "2":
                    self.modelTimeOptions.disableItem(str_model=itm)
                elif str(itm) == "1" and model == 'two_way':
                    self.modelTimeOptions.disableItem(str_model=itm)

        # Constraints on time step from compressible model

        from code_saturne.model.CompressibleModel import CompressibleModel
        model = CompressibleModel(self.case).getCompressibleModel()
        if model != 'off':
            if idtvar not in [0, 1]:
                idtvar = 0

            # Deactivate steady time step
            # Using list to allow additions in the future
            for itm in self.modelTimeOptions.getItems():
                if str(itm) in ["2"]:
                    self.modelTimeOptions.disableItem(str_model=itm)

            self.labelNTERUP.setText("Velocity-Pressure algorithm\nsub-iterations on Navier-Stokes")
            self.comboBoxNTERUP.hide()
            self.spinBoxNTERUP.show()

        # Constraints on time step from groundwater model

        from code_saturne.model.GroundwaterModel import GroundwaterModel
        model = GroundwaterModel(self.case).getGroundwaterModel()
        if model != 'off':
            if idtvar not in [0, 1]:
                idtvar = 0

            # Deactivate all modes except constant time step
            for itm in self.modelTimeOptions.getItems():
                if str(itm) != "0":
                    self.modelTimeOptions.disableItem(str_model=itm)

        # Change time step option if required by model constraints

        if idtvar_p != idtvar:
            self.mdl.setTimePassing(idtvar)

        if self.mdl.thermalCase():
            if self.mdl.getThermalTimeStep() == 'on':
                self.checkBoxIPTLRO.setChecked(True)
            else:
                self.checkBoxIPTLRO.setChecked(False)
        else:
            self.labelIPTLRO.hide()
            self.checkBoxIPTLRO.hide()
            self.mdl.RemoveThermalTimeStepNode()

        self.__setTimePassingDisplay(idtvar)
        self.__setStopCritDisplay()

        self.case.undoStartGlobal()


    def __setTimePassingDisplay(self, idtvar):
        """
        Choices based on IDTVAR.
        """

        self.modelTimeOptions.setItem(str_model=str(idtvar))

        if idtvar in (-1, 2):
            if idtvar == -1:
                self.modelNTERUP.enableItem(str_model = 'simple')
                self.modelNTERUP.disableItem(str_model = 'simplec')
            elif idtvar == 2:
                self.modelNTERUP.disableItem(str_model = 'simple')
                self.modelNTERUP.enableItem(str_model = 'simplec')
            self.modelNTERUP.disableItem(str_model = 'piso')

            m_prev, c_prev = self.mdl.getStopCriterion()
            for m in ('maximum_time', 'maximum_time_add'):
                if m_prev == m:
                    dtref = self.mdl.getTimeStep()
                    value = int(float(c_prev) / float(dtref))
                    model = 'iterations'
                    if m[-4:] == '_add':
                        model += '_add'
                    self.mdl.setStopCriterion(model, value)
                    self.__setStopCritDisplay()

                self.modelTimeStop.disableItem(str_model=m)

        else:
            self.modelNTERUP.disableItem(str_model = 'simple')
            self.modelNTERUP.enableItem(str_model = 'simplec')
            self.modelNTERUP.enableItem(str_model = 'piso')

            self.modelTimeStop.enableItem(str_model='maximum_time')
            self.modelTimeStop.enableItem(str_model='maximum_time_add')

        algo = self.mdl.getVelocityPressureAlgorithm()
        self.modelNTERUP.setItem(str_model=algo)
        if algo == 'piso':
            self.spinBoxNTERUP.show()
        else:
            self.spinBoxNTERUP.hide()

        value = self.mdl.getVelocityPressureParamSweepNumber()
        self.spinBoxNTERUP.setValue(value)

        if idtvar == -1:
            self.labelRELXST.show()
            self.lineEditRELXST.show()
            self.labelDTREF.hide()
            self.labelDTREFunit.hide()
            self.lineEditDTREF.hide()
            relax_coef = self.mdl.getRelaxCoefficient()
            self.lineEditRELXST.setText(str(relax_coef))

        else:
            self.labelRELXST.hide()
            self.lineEditRELXST.hide()
            self.labelDTREF.show()
            self.labelDTREFunit.show()
            self.lineEditDTREF.show()
            dtref = self.mdl.getTimeStep()
            self.lineEditDTREF.setText(str(dtref))

        if idtvar in (1, 2):
            courant_max   = self.mdl.getOptions('max_courant_num')
            fourier_max   = self.mdl.getOptions('max_fourier_num')
            time_step_min_factor = self.mdl.getOptions('time_step_min_factor')
            time_step_max_factor = self.mdl.getOptions('time_step_max_factor')
            time_step_var = self.mdl.getOptions('time_step_var')

            self.lineEditCOUMAX.setText(str(courant_max))
            self.lineEditFOUMAX.setText(str(fourier_max))
            self.lineEditCDTMIN.setText(str(time_step_min_factor))
            self.lineEditCDTMAX.setText(str(time_step_max_factor))
            self.lineEditVARRDT.setText(str(time_step_var))

            self.labelCOUMAX.show()
            self.lineEditCOUMAX.show()
            self.labelFOUMAX.show()
            self.lineEditFOUMAX.show()
            self.labelCDTMIN.show()
            self.lineEditCDTMIN.show()
            self.labelCDTMAX.show()
            self.lineEditCDTMAX.show()
            self.labelVARRDT.show()
            self.lineEditVARRDT.show()

        else:
            self.labelCOUMAX.hide()
            self.lineEditCOUMAX.hide()
            self.labelFOUMAX.hide()
            self.lineEditFOUMAX.hide()
            self.labelCDTMIN.hide()
            self.lineEditCDTMIN.hide()
            self.labelCDTMAX.hide()
            self.lineEditCDTMAX.hide()
            self.labelVARRDT.hide()
            self.lineEditVARRDT.hide()


    def __setStopCritDisplay(self):
        """
        Stop criterion option
        """

        model, value = self.mdl.getStopCriterion()

        if model in ("iterations", "iterations_add"):
            self.lineEditStop.setValidator(self.validatorNTABS)
        elif model in ("maximum_time", "maximum_time_add"):
            self.lineEditStop.setValidator(self.validatorTABS)

        self.modelTimeStop.setItem(str_model=str(model))
        self.lineEditStop.setText(str(value))


    @pyqtSlot(str)
    def slotTimePassing(self, text):
        """
        Input IDTVAR.
        """
        idtvar = int(self.modelTimeOptions.dicoV2M[str(text)])

        self.mdl.setTimePassing(idtvar)

        self.__setTimePassingDisplay(idtvar)


    @pyqtSlot(str)
    def slotStopCritModel(self, text):
        """
        Select stop criterion model
        """
        m_prev, c_prev = self.mdl.getStopCriterion()
        model = self.modelTimeStop.dicoV2M[str(text)]

        value = c_prev
        if m_prev in ("iterations", "iterations_add"):
            if model in ("maximum_time", "maximum_time_add"):
                dtref = self.mdl.getTimeStep()
                value = float(c_prev) * float(dtref)
        elif m_prev in ("maximum_time", "maximum_time_add"):
            if model in ("iterations", "iterations_add"):
                dtref = self.mdl.getTimeStep()
                value = int(float(c_prev) / float(dtref))

        self.mdl.setStopCriterion(model, value)

        self.__setStopCritDisplay()


    @pyqtSlot(str)
    def slotStopCritValue(self, text):
        """
        Input stop criterion
        """
        if self.lineEditStop.validator().state == QValidator.Acceptable:

            model, c_prev = self.mdl.getStopCriterion()

            value = c_prev
            if model in ("iterations", "iterations_add"):
                value = from_qvariant(text, int)
                self.mdl.setStopCriterion(model, value)
            elif model in ("maximum_time", "maximum_time_add"):
                value = from_qvariant(text, float)
                self.mdl.setStopCriterion(model, value)


    @pyqtSlot(str)
    def slotTimeStep(self, text):
        """
        Input DTREF.
        """
        if self.lineEditDTREF.validator().state == QValidator.Acceptable:
            time_step = from_qvariant(text, float)
            self.mdl.setTimeStep(time_step)


    @pyqtSlot(str)
    def slotNTERUP(self,text):
        """
        Set value for parameterNTERUP
        """
        NTERUP = self.modelNTERUP.dicoV2M[str(text)]
        self.mdl.setVelocityPressureAlgorithm(NTERUP)
        if NTERUP == 'piso':
            self.spinBoxNTERUP.show()
            value = self.mdl.getVelocityPressureParamSweepNumber()
            self.spinBoxNTERUP.setValue(value)
        else:
            self.spinBoxNTERUP.hide()
        self.browser.configureTree(self.case)
        log.debug("slotNTERUP-> %s" % NTERUP)


    @pyqtSlot(int)
    def slotNTERUP2(self, var):
        """
        Set value for velocity-pressure parameter sweep number
        """
        self.mdl.setVelocityPressureParamSweepNumber(var)
        log.debug("slotNTERUP2-> %s" % var)


    @pyqtSlot(str)
    def slotRelaxCoef(self, text):
        """
        Input relaxation coefficient.
        """
        if self. lineEditRELXST.validator().state == QValidator.Acceptable:
            relax_coef = from_qvariant(text, float)
            self.mdl.setRelaxCoefficient(relax_coef)


    @pyqtSlot(str)
    def slotTimeOptionCOUMAX(self, text):
        """
        Input COUMAX.
        """
        if self.lineEditCOUMAX.validator().state == QValidator.Acceptable:
            courant_max = from_qvariant(text, float)
            self.mdl.setOptions('max_courant_num', courant_max)


    @pyqtSlot(str)
    def slotTimeOptionFOUMAX(self, text):
        """
        Input FOUMAX.
        """
        if self.lineEditFOUMAX.validator().state == QValidator.Acceptable:
            fourier_max = from_qvariant(text, float)
            self.mdl.setOptions('max_fourier_num', fourier_max)


    @pyqtSlot(str)
    def slotTimeOptionCDTMIN(self, text):
        """
        Input CDTMIN.
        """
        if self.lineEditCDTMIN.validator().state == QValidator.Acceptable:
            time_step_min_factor = from_qvariant(text, float)
            self.mdl.setOptions('time_step_min_factor', time_step_min_factor)


    @pyqtSlot(str)
    def slotTimeOptionCDTMAX(self, text):
        """
        Input CDTMAX.
        """
        if self.lineEditCDTMAX.validator().state == QValidator.Acceptable:
            time_step_max_factor = from_qvariant(text, float)
            self.mdl.setOptions('time_step_max_factor', time_step_max_factor)


    @pyqtSlot(str)
    def slotTimeOptionVARRDT(self, text):
        """
        Input VARRDT.
        """
        if self.lineEditVARRDT.validator().state == QValidator.Acceptable:
            time_step_var = from_qvariant(text, float)
            self.mdl.setOptions('time_step_var', time_step_var)


    @pyqtSlot()
    def slotThermalTimeStep(self):
        """
        Input IPTLRO.
        """
        if self.checkBoxIPTLRO.isChecked():
            self.mdl.setThermalTimeStep("on")
        else:
            self.mdl.setThermalTimeStep("off")


#-------------------------------------------------------------------------------
# Testing part
#-------------------------------------------------------------------------------

if __name__ == "__main__":
    pass

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
Output of volume variables
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import unittest

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import *
from code_saturne.model.XMLmodel import XMLmodel, ModelTest
from code_saturne.model.XMLvariables import Model, Variables
from code_saturne.model.DefineUserScalarsModel import DefineUserScalarsModel
from code_saturne.model.ThermalRadiationModel import ThermalRadiationModel

#-------------------------------------------------------------------------------
# Model class
#-------------------------------------------------------------------------------

class OutputVolumicVariablesModel(Variables, Model):

    def __init__(self, case):
        """
        Constuctor.
        """
        self.case = case
        self.node_models    = self.case.xmlGetNode('thermophysical_models')
        self.analysis_ctrl  = self.case.xmlGetNode('analysis_control')
        self.fluid_prop     = self.case.xmlGetNode('physical_properties')
        self.have_multizone_phys_prop = self.__has_multiple_phys_prop_nodes__()

        if self.node_models:
            self.node_model_vp  = self.node_models.xmlGetNode('velocity_pressure')
            self.node_ale       = self.node_models.xmlGetChildNode('ale_method')
        else:
            self.node_model_vp  = None
            self.node_ale       = None
        if self.analysis_ctrl:
            self.node_output    = self.analysis_ctrl.xmlInitNode('output')
            self.node_means     = self.analysis_ctrl.xmlGetNode('time_averages')
            self.node_error     = self.analysis_ctrl.xmlGetNode('error_estimator')
        else:
            self.node_output    = None
            self.node_means     = None
            self.node_error     = None
        if self.node_output:
            self.node_probe     = self.node_output.xmlGetNodeList('probe','name')
        else:
            elf.node_probe     = None

        self.node_lagr_vstats = None
        node_lagr  = self.case.xmlGetNode('lagrangian', 'model')
        if node_lagr:
            node_stat = node_lagr.xmlGetChildNode('statistics')
            if node_stat:
                self.node_lagr_vstats = node_stat.xmlGetChildNode('volume')

        self.updateList()


    # Following private methods: (to see for gathering eventually)


    def __has_multiple_phys_prop_nodes__(self):
        """
        Determine if we have mutiple physical properties nodes
        """
        # We do not use the LocalizationModel here, as it would lead to
        # circular dependencies, so directly use XML definitions.
        m_pp_zones = 0
        n = self.case.xmlGetNode('solution_domain')
        if n:
            n = n.xmlGetNode('volumic_conditions')
            if n:
                nl = n.xmlGetChildNodeList('zone', 'label', 'id')
                for n in nl:
                    if n['physical_properties'] == 'on':
                        m_pp_zones += 1
                        if m_pp_zones > 1:
                            return True
        return False


    def _defaultValues(self):
        """
        Return in a dictionnary which contains default values
        """
        default = {}
        default['status']    = "on"
        default['estimator'] = "0"

        return default


    def __getVolFieldsNodeList(self, root, constant, time_averages):
        """
        Returns the list of active volume fields (variables, propeties,
        and possibly postprocessing fields).
        """

        tags = ['variable', 'property', 'scalar']
        if time_averages:
            tags.append('time_average')

        l = []
        for tag in tags:
            for node in root.xmlGetNodeList(tag):
                if node['support']:
                    if node['support'] == 'boundary':
                        continue
                if tag == 'property':
                    if not (self.have_multizone_phys_prop or constant):
                        choice = node['choice']
                        if choice and choice == 'constant':
                            continue
                if node.xmlGetAttribute('status', default='on') == 'on':
                    l.append(node)

        return l


    def getVolumeFieldsNodeList(self, constant=False, time_averages=True):
        """
        Returns the list of active volume fields (variables, properties,
        and possibly postprocessing fields).
        """

        l = self.__getVolFieldsNodeList(self.case, constant, time_averages)

        n_bc = self.case.xmlGetNode('boundary_conditions')
        if n_bc:
            l_c = self.__getVolFieldsNodeList(n_bc, constant, time_averages)
            for n in l_c:
                l.remove(n)

        return l


    def getVolumeFieldsLabel2Name(self,
                                  constant=False,
                                  time_averages=True,
                                  get_components=False):
        """
        Returns a dictionnary to relate name and label to volume fields
        (variables, properties) for time averages and profiles.
        """

        dicoLabel2Name = {}

        for node in self.getVolumeFieldsNodeList(constant, time_averages):

            name = self.__nodeName__(node)
            label = node['label']
            if not label:
                label = name

            # Check that we wish to show the field
            if not (node['support'] and node['support'] == "boundary"):
                # Add main field. "-1" is used as default since for
                # vectors/tensors it yields all components
                dicoLabel2Name[label] = (name, str(-1))

                # Check if we want to also get components
                dim = node['dimension']
                if dim and int(dim) > 1 and get_components:
                    # If we consider the Rij tensor, the user will see
                    # R11, R22, ... in the GUI instead of Rij[0], Rij[1], ...
                    # This choice was considered as the clearest.
                    if name == 'rij':
                        rij_lbls = ['R11', 'R22', 'R33', 'R12', 'R23', 'R13']
                        for ii in range(int(dim)):
                            label1 = rij_lbls[ii]
                            dicoLabel2Name[label1] = (name, str(ii))

                    elif 'reynolds_stress' in name:
                        comp = label.split('reynolds_stress')[-1]
                        rij_lbls = ['reynolds_stress11',
                                    'reynolds_stress22',
                                    'reynolds_stress33',
                                    'reynolds_stress12',
                                    'reynolds_stress23',
                                    'reynolds_stress13']
                        for ii in range(int(dim)):
                            label1 = rij_lbls[ii] + comp
                            dicoLabel2Name[label1] = (name, str(ii))

                    else:
                        for ii in range(int(dim)):
                            label1 = label + "[" + str(ii) + "]"
                            dicoLabel2Name[label1] = (name, str(ii))

                is_scalar = not( (dim != None) and (int(dim)>1) )
                if is_scalar:
                    dicoLabel2Name[label] = (name, None) 

        return dicoLabel2Name

    def getVariablesAtNode(self, node, time_averages=False, get_components=False):
        recognized_variables = []
        dicoLabel2Name =self.getVolumeFieldsLabel2Name(time_averages=time_averages, get_components=get_components)
        for xml_variable in node.xmlGetChildNodeList('var_prop'):
            for name, label in dicoLabel2Name.items():
                component = label[1]
                is_recognized = (label[0] == xml_variable['name'])
                if component:
                    is_recognized = (label == (xml_variable['name'], xml_variable['component']))
                if is_recognized:
                    recognized_variables.append(name)
        return recognized_variables

    def setVariablesAtNode(self, node, variables, time_averages=False, get_components=False):
        node.xmlRemoveChild('var_prop')
        dicoLabel2Name = self.getVolumeFieldsLabel2Name(time_averages=time_averages,
                                      get_components=get_components)
        authorized_variables = dicoLabel2Name.keys()
        for var in variables:
            self.isInList(var, authorized_variables)
            (name, comp) = dicoLabel2Name[var]
            if comp == None:
                comp = -1
            node.xmlAddChild('var_prop', name=name, component=comp)

    def __updateListFilter(self, node_list, category, remain_list=None):
        """
        Update name list using a location filter
        """
        self.listNodeVolum = self.getVolumeFieldsNodeList()

        list_nodes = self.getVolumeFieldsNodeList()

        # Main/known categories

        for node in node_list:
            if node['support']:
                if node['support'] == 'boundary':
                    continue
            elif node.xmlGetAttribute('status', default='on') == 'off':
                continue
            name = self.__nodeName__(node)
            label = node['label']
            if not label:
                label = name
            self.dicoLabelName[name] = label
            self.list_name.append([self.__nodeName__(node), category])

            if remain_list:
                for node in node_list:
                    try:
                        remain_list.remove(node)
                    except Exception:
                        pass


    def updateList(self):
        """
        Update list of volume nodes and matching names and category
        """
        self.listNodeVolum = self.getVolumeFieldsNodeList()

        list_remain = self.getVolumeFieldsNodeList()

        self.dicoLabelName = {}
        self.list_name = []

        fields_node = None
        fields_list = []

        # Main/known categories

        self.__updateListFilter(self.__getListOfVelocityPressureVariables__(),
                                'Base', list_remain)
        self.__updateListFilter(self.__getListOfTurbulenceVariables__(),
                                'Turbulence', list_remain)
        self.__updateListFilter(self.getThermalScalar(),
                                'Thermal', list_remain)
        self.__updateListFilter(self.__getThermalRadiativeProperties__(),
                                'Thermal', list_remain)
        self.__updateListFilter(self.__getPuCoalScalProper__(),
                                'Coal', list_remain)
        self.__updateListFilter(self.__getGasCombScalProper__(),
                                'Gas', list_remain)
        self.__updateListFilter(self.__getMeteoScalProper__(),
                                'Atmospheric', list_remain)
        self.__updateListFilter(self.__getElecScalProper__(),
                                'Electric', list_remain)
        self.__updateListFilter(self.getAdditionalScalar(),
                                'Other', list_remain)
        self.__updateListFilter(self.__getAdditionalScalarProperty__(),
                                'Other', list_remain)
        self.__updateListFilter(self.__getFluidProperty__(),
                                'Physical properties', list_remain)
        self.__updateListFilter(self.__getTimeProperty__(),
                                'Other', list_remain)
        self.__updateListFilter(self.__getListOfTimeAverage__(),
                                'Time moments', list_remain)
        self.__updateListFilter(self.__getListOfLagrVolumeStats__(),
                                'Lagrangian statistics', list_remain)
        self.__updateListFilter(self.__getListOfAleMethod__(),
                                'Other', list_remain)
        self.__updateListFilter(self.__getListOfEstimator__(),
                                'Estimator', list_remain)

        # Remaining nodes

        for node in list_remain:

            # location filter

            if node['support']:
                if node['support'] == 'boundary':
                    continue

            # Name and label

            if node['name']:
                name = self.__nodeName__(node)
            else:
                name = node['label']
            if not name:
                continue
            if not node['label']:
                node['label'] = name

            self.dicoLabelName[name] = node['label']

            # Category based on parent node
            # (could also be based on dictionary)

            category = node.xmlGetParentName()
            if category == 'fluid_properties':
                category = 'Physical properties'
            else:
                category = category.replace('_', ' ').capitalize()

            # For NCFD multiphase, use the field_id as a parent category
            if node['field_id']:
                if node['field_id'] != 'none':
                    if fields_node == None:
                        fields_node = self.node_models.xmlGetNode('fields')
                        for nf in fields_node.xmlGetNodeList('field'):
                            fields_list.append(nf['label'])

                    category = fields_list[int(node['field_id'])-1]



            self.list_name.append([self.__nodeName__(node), category])


    def __getNode__(self, name):
        """
        Return a node matching a given name
        """

        for node in self.listNodeVolum:
            n_name = node['name']
            if node['field_id']:
                if node['field_id'] != 'none':
                    n_name += '_' + str(node['field_id'])
            if n_name == name:
                return node

        return None


    def __nodeName__(self, node):
        """
        Return a node matching a given name
        """

        n_name = node['name']
        if not n_name:
            n_name = node['label']
        else:
            if node['field_id']:
                if node['field_id'] != 'none':
                    n_name += '_' + str(node['field_id'])

        return n_name


    def __getListOfVelocityPressureVariables__(self):
        """
        Private method: return node of properties of weight matrix
        """
        nodeList = []
        if self.node_model_vp:
            for tag in ('variable', 'property'):
                for node in self.node_model_vp.xmlGetNodeList(tag):
                    nodeList.append(node)
        return nodeList


    def __getListOfTurbulenceVariables__(self):
        """
        Private method: return node of properties of weight matrix
        """
        node_models = self.case.xmlGetNode('thermophysical_models')
        if node_models:
            node_turb = node_models.xmlGetNode('turbulence', 'model')

        nodeList = []
        if node_turb:
            for tag in ('variable', 'property'):
                for node in node_turb.xmlGetNodeList(tag):
                    if node['name'] == 'rij':
                        node['dimension'] = 6
                    nodeList.append(node)
        return nodeList


    def __getListOfEstimator__(self):
        """
        Private method: return node of properties of weight matrix
        """
        nodeList = []
        if self.node_error:
            for node in self.node_error.xmlGetNodeList('property'):
                nodeList.append(node)
        return nodeList


    def __getWeightMatrixProperty__(self):
        """
        Private method: return node of properties of weight matrix
        """
        nodeList = []

        node0 = self.case.xmlGetNode('numerical_parameters')
        if node0:
            node1 = node0.xmlGetNode('velocity_pressure_coupling', 'status')
            if node1:
                if node1['status'] == 'on':
                    nodeList = node0.xmlGetNodeList('property')
        return nodeList


    def __getListOfAleMethod__(self):
        """
        Private method: return list of variables and properties for ALE method
        """
        nodeList = []
        if self.node_ale:
            if self.node_ale['status'] == 'on':
                for tag in ('variable', 'property'):
                    for node in self.node_ale.xmlGetChildNodeList(tag):
                        nodeList.append(node)

        return nodeList


    def __getThermalRadiativeProperties__(self):
        """
        Private method: return list of volumic properties for thermal radiation
        """
        nodeList = []
        if ThermalRadiationModel(self.case).getRadiativeModel() != "off":
            self.node_ray = self.node_models.xmlGetNode('radiative_transfer')
            for node in self.node_ray.xmlGetChildNodeList('property'):
                nodeList.append(node)
        return nodeList


    @Variables.noUndo
    def getThermalScalar(self):
        """
        Return node of thermal scalar (idem ds NumericalParamEquationModel)
        """
        node_models = self.case.xmlGetNode('thermophysical_models')
        node = node_models.xmlGetNode('thermal_scalar')
        if node:
            return node.xmlGetNodeList('variable', type='thermal')
        else:
            return []


    @Variables.noUndo
    def __getPuCoalScalProper__(self):
        """
        Return list fo nodes of pulverized coal.
        """
        varList = []
        if self.node_models:
            node = self.node_models.xmlGetNode('solid_fuels', 'model')
            if node:
                model = node['model']
                if model != 'off':
                    for var in ('variable', 'property'):
                        nodList = node.xmlGetNodeList(var)
                        for nodvar in nodList:
                            varList.append(nodvar)
        return varList


    @Variables.noUndo
    def __getGasCombScalProper__(self):
        """
        Return list of nodes of gas combustion.
        """
        varList = []
        if self.node_models:
            node = self.node_models.xmlGetNode('gas_combustion', 'model')
            if node:
                model = node['model']
                if model != 'off':
                    for var in ('variable', 'property'):
                        nodList = node.xmlGetNodeList(var)
                        for nodvar in nodList:
                            varList.append(nodvar)
        return varList


    @Variables.noUndo
    def __getMeteoScalProper__(self):
        """
        Return list fo nodes of atmospheric flows.
        """
        varList = []
        if self.node_models:
            node = self.node_models.xmlGetNode('atmospheric_flows', 'model')
            if node:
                model = node['model']
                if model != 'off':
                    for var in ('variable', 'property'):
                        nodList = node.xmlGetNodeList(var)
                        for nodvar in nodList:
                            varList.append(nodvar)
        return varList


    @Variables.noUndo
    def __getElecScalProper__(self):
        """
        Return list fo nodes of electric flows.
        """
        varList = []
        if self.node_models:
            node = self.node_models.xmlGetNode('joule_effect', 'model')
            if not node: return []
            model = node['model']
            if model != 'off':
                for var in ('variable', 'property'):
                    nodList = node.xmlGetNodeList(var)
                    for nodvar in nodList:
                        varList.append(nodvar)
        return varList


    @Variables.noUndo
    def getModelVariables(self, model_name):
        """
        Return list of variable nodes for a given model.
        """
        varList = []
        if self.node_models:
            node = self.node_models.xmlGetNode(model_name, 'model')
            if node:
                model = node['model']
                if model != 'off':
                    nodList = node.xmlGetNodeList('variable')
                    for nodvar in nodList:
                        varList.append(nodvar)
        return varList


    @Variables.noUndo
    def getAdditionalScalar(self):
        """
        Return list of nodes of user scalars
        """
        node = self.case.xmlGetNode('additional_scalars')
        if node:
            return node.xmlGetNodeList('variable', type='user')
        else:
            return []


    @Variables.noUndo
    def __getAdditionalScalarProperty__(self):
        """
        Return list of nodes of properties of user scalars
        """
        nodeList = []
        for node in self.getAdditionalScalar():
            L = node.xmlGetNode('property', choice='variable')
            if L:
                nodeList.append(L)
        return nodeList


    @Variables.noUndo
    def __getFluidProperty__(self):
        """
        Return list of nodes of fluid properties
        """
        nodeList = []

        if self.fluid_prop:
            node = self.fluid_prop.xmlGetNode('fluid_properties')

            if node:
                for prop in ('density',
                             'specific_heat',
                             'thermal_conductivity'):
                    L = node.xmlGetNode('property', name=prop, choice='variable')
                    if not L:
                        L = node.xmlGetNode('property', name=prop, choice='user_law')
                    if self.have_multizone_phys_prop and not L:
                        L = node.xmlGetNode('property', name=prop, choice='constant')
                    if L:
                        nodeList.append(L)
                for prop in ('molecular_viscosity',
                             'dynamic_diffusion'):
                    L = node.xmlGetNode('property', name=prop, choice='variable')
                    if not L:
                        L = node.xmlGetNode('property', name=prop, choice='user_law')
                    if L:
                        nodeList.append(L)

        return nodeList


    @Variables.noUndo
    def __getTimeProperty__(self):
        """
        Return list fo nodes of properties of time_parameters.
        """
        nodeList = []

        if self.analysis_ctrl:
            node1 = self.analysis_ctrl.xmlGetNode('time_parameters')
        else:
            node1 = None

        if node1:
            if node1.xmlGetInt('time_passing'):
                node2 = node1.xmlGetNode('property', name='local_time_step')
                if node2:
                    nodeList.append(node2)

            for prop in ('courant_number', 'fourier_number'):
                L = node1.xmlGetNode('property', name=prop)
                if L: nodeList.append(L)

        return nodeList


    @Variables.noUndo
    def __getListOfTimeAverage__(self):
        """
        Return list of time averages variables
        """
        nodeList = []
        if self.node_means:
            for node in self.node_means.xmlGetNodeList('time_average'):
                nodeList.append(node)

        return nodeList


    def __getListOfLagrVolumeStats__(self):
        """
        Private method: return node of properties of weight matrix
        """
        nodeList = []
        if self.node_lagr_vstats:
            for node in self.node_lagr_vstats.xmlGetNodeList('property'):
                nodeList.append(node)

        return nodeList


    #Following methods only called by the View
    @Variables.noUndo
    def getLabelsList(self):
        """
        Return list of labels for all variables, properties .....Only for the View
        """
        lst = []
        for node in self.listNodeVolum:
            lst.append(node['label'])
        return lst


    @Variables.noUndo
    def getNamesList(self):
        """
        Return list of names for all variables, properties .....Only for the View
        """
        lst = []
        for node in self.listNodeVolum:
            lst.append(self.__nodeName__(node))
        return lst


    @Variables.noUndo
    def getProbeList(self):
        """ Return list of node for probes """
        probeList = []
        for node in self.node_probe:
            probeList.append(self.__nodeName__(node))
        return probeList


    @Variables.noUndo
    def getPrintingStatus(self, name):
        """
        Return status of markup printing from node with name. Only for the View
        """
        self.isInList(name, self.getNamesList())
        status = self._defaultValues()['status']
        node = self.__getNode__(name)
        if node:
            node_printing = node.xmlGetChildNode('listing_printing', 'status')
            if node_printing:
                status = node_printing['status']
        return status


    @Variables.noUndo
    def getPostStatus(self, name):
        """
        Return status of markup  post processing from node with name. Only for the View
        """
        self.isInList(name, self.getNamesList())
        status = self._defaultValues()['status']
        node = self.__getNode__(name)
        if node:
            node_post = node.xmlGetChildNode('postprocessing_recording', 'status')
            if node_post:
                status = node_post['status']
        return status


    @Variables.noUndo
    def getMonitorStatus(self, name):
        """
        Return status of markup monitoring from node with name. Only for the View
        """
        self.isInList(name, self.getNamesList())
        status = self._defaultValues()['status']
        node = self.__getNode__(name)
        if node:
            node_post = node.xmlGetChildNode('probes_recording', 'status')
            if node_post:
                status = node_post['status']
        return status


    @Variables.undoLocal
    def setVariableLabel(self, old_label, new_label):
        """
        Replace old_label by new_label for node with name and old_label.
        Only for the View
        """
        # fusion de cette methode avec DefineUserScalarsModel.renameScalarLabel
        self.isInList(old_label, self.getLabelsList())
        self.isNotInList(new_label, [""])

        if old_label != new_label:
            self.isNotInList(new_label, self.getLabelsList())
        for node in self.listNodeVolum:
            if node['label'] == old_label:
                node['label'] = new_label

        self.updateList()
        self._updateBoundariesNodes(old_label, new_label)

        for node in self.case.xmlGetNodeList('formula'):
            f = node.xmlGetTextNode()
            if f:
                f.replace(old_label, new_label)
                node.xmlSetTextNode(f)


    def _updateBoundariesNodes(self, old_label, new_label):
        """
        Update good label for boundaries nodes with name and label.
        Only for the View
        """
        self.node_bc  = self.case.xmlInitNode('boundary_conditions')
        self.node_var = self.node_bc.xmlInitNodeList('variable')

        for node in self.node_var:
            if node['label'] == old_label:
                node['label'] = new_label


    @Variables.undoLocal
    def setPrintingStatus(self, name, status):
        """
        Put status for printing from node with name and label
        """
        self.isOnOff(status)
        self.isInList(name, self.getNamesList())
        node = self.__getNode__(name)
        if node:
            if status == 'off':
                node.xmlInitChildNode('listing_printing')['status'] = status
            else:
                if node.xmlGetChildNode('listing_printing'):
                    node.xmlRemoveChild('listing_printing')


    @Variables.noUndo
    def getVariableLabel(self, name) :
        """
        return label of name variable
        """
        for variableType in ('variable', 'property') :
            node = self.case.xmlGetNode(variableType, name = name)
            if node != None:
                break

        if node != None:
            label = node['label']
            return label
        else :
            msg = "This variable " + name + " doesn't exist"
            raise ValueError(msg)


    @Variables.undoLocal
    def setPostStatus(self, name, status):
        """
        Put status for postprocessing from node with name and label
        """
        self.isOnOff(status)
        self.isInList(name, self.getNamesList())
        self.isInList(name, self.getNamesList())
        node = self.__getNode__(name)
        if node:
            if status == 'off':
                node.xmlInitChildNode('postprocessing_recording')['status'] = status
            else:
                if node.xmlGetChildNode('postprocessing_recording'):
                    node.xmlRemoveChild('postprocessing_recording')


    @Variables.undoLocal
    def setMonitorStatus(self, name, status):
        """
        Put status for monitoring from node with name and label
        """
        self.isOnOff(status)
        self.isInList(name, self.getNamesList())
        node = self.__getNode__(name)
        if node:
            if status == 'off':
                node.xmlInitChildNode('probes_recording')['status'] = status
            else:
                if node.xmlGetChildNode('probes_recording'):
                    node.xmlRemoveChild('probes_recording')


    @Variables.noUndo
    def getEstimatorModel(self, name):
        """
        Return model for an error estimator
        """
        self.isInList(name, ["Correction", "Drift", "Prediction", "Total"])
        status = self._defaultValues()['estimator']

        if self.node_error:
            nn = self.node_error.xmlGetChildNode(name, 'model')
            if nn:
                status = nn['model']

        return status


    @Variables.undoLocal
    def setEstimatorModel(self, name, model):
        """
        Put model for an error estimator
        """
        self.isInList(model, ['0', '1', '2'])
        self.isInList(name, ["Correction", "Drift", "Prediction", "Total"])
        status = self._defaultValues()['estimator']

        if not self.analysis_ctrl:
            self.analysis_ctrl = self.case.xmlInitNode('analysis_control')
        if not self.node_error:
            self.node_error = self.analysis_ctrl.xmlInitNode('error_estimator')

        if model != status:
            if self.node_error.xmlGetChildNode(name):
                self.node_error.xmlRemoveChild(name)
            self.node_error.xmlInitChildNode(name)['model'] = model
        else:
            if self.node_error.xmlGetChildNode(name):
                self.node_error.xmlRemoveChild(name)

        # add and remove field associated
        if model != status:
            nn = self.node_error.xmlGetNode(name)
            if name == "Correction":
                self.setNewProperty(nn, "est_error_cor_" + model)
            elif name == "Drift":
                self.setNewProperty(nn, "est_error_der_" + model)
            elif name == "Prediction":
                self.setNewProperty(nn, "est_error_pre_" + model)
            elif name == "Total":
                self.setNewProperty(nn, "est_error_tot_" + model)

        self.updateList()


#-------------------------------------------------------------------------------
# OutputVolumicVariablesModel Test Class
#-------------------------------------------------------------------------------

class OutputVolumicVariablesModelTestCase(ModelTest):
    """
    Unittest
    """
    def checkOutputVolumicVariablesModelInstantiation(self):
        """Check whether the OutputVolumicVariablesModel class could be instantiated"""
        mdl = None
        mdl = OutputVolumicVariablesModel(self.case)
        assert mdl != None, 'Could not instantiate OutputVolumicVariablesModel'


    def checkSetVariableLabel(self):
        """
        Check whether the OutputVolumicVariablesModel class could be set a label
        of property
        """
        model = OutputVolumicVariablesModel(self.case)
        model.setVariableLabel('VelocitV', 'vitV')
        node = model.node_models.xmlInitNode('velocity_pressure')
        doc = '''<velocity_pressure>
                    <variable label="Pressure" name="pressure"/>
                    <variable label="VelocitU" name="velocity_U"/>
                    <variable label="vitV" name="velocity_V"/>
                    <variable label="VelocitW" name="velocity_W"/>
                    <property label="total_pressure" name="total_pressure"/>
                    <property label="Yplus" name="yplus" support="boundary"/>
                    <property label="Stress" name="stress" support="boundary"/>
                 </velocity_pressure>'''
        assert node == self.xmlNodeFromString(doc),\
            'Could not set label of property in output volumic variables model'

    def checkSetAndGetPrintingStatus(self):
        """
        Check whether the OutputVolumicVariablesModel class could be
        set and get status for printing listing
        """
        from code_saturne.model.ThermalScalarModel import ThermalScalarModel
        ThermalScalarModel(self.case).setThermalModel('temperature_celsius')
        del ThermalScalarModel

        mdl = OutputVolumicVariablesModel(self.case)
        mdl.setPrintingStatus('TempC', 'off')
        node_out = mdl.case.xmlGetNode('additional_scalars')
        doc = '''<additional_scalars>
                    <variable label="TempC" name="temperature_celsius" type="thermal">
                        <initial_value zone_id="1">20.0</initial_value>
                        <min_value>-1e+12</min_value>
                        <max_value>1e+12</max_value>
                        <listing_printing status="off"/>
                    </variable>
                 </additional_scalars>'''

        assert node_out == self.xmlNodeFromString(doc),\
            'Could not set status of listing printing in output volumic variables model'
        assert mdl.getPrintingStatus('TempC') == 'off',\
            'Could not get status of listing printing in output volumic variables model'

    def checkSetAndGetPostStatus(self):
        """
        Check whether the OutputVolumicVariablesModel class could be
        set and get status for printing
        """
        from code_saturne.model.ThermalScalarModel import ThermalScalarModel
        ThermalScalarModel(self.case).setThermalModel('temperature_celsius')
        del ThermalScalarModel

        mdl = OutputVolumicVariablesModel(self.case)
        mdl.setPostStatus('TempC', 'off')
        node_out = mdl.case.xmlGetNode('additional_scalars')
        doc = '''<additional_scalars>
                    <variable label="TempC" name="temperature_celsius" type="thermal">
                        <initial_value zone_id="1">20.0</initial_value>
                        <min_value>-1e+12</min_value>
                        <max_value>1e+12</max_value>
                        <postprocessing_recording status="off"/>
                    </variable>
                 </additional_scalars>'''

        assert node_out == self.xmlNodeFromString(doc),\
            'Could not set status of post processing in output volumic variables model'
        assert mdl.getPostStatus('TempC') == 'off',\
            'Could not get status of post processing in output volumic variables model'

def suite():
    testSuite = unittest.makeSuite(OutputVolumicVariablesModelTestCase, "check")
    return testSuite

def runTest():
    print("OutputVolumicVariablesModelTestCase")
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# -------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

# -------------------------------------------------------------------------------

"""
This module defines the Porosity model data management.

This module contains the following classes:
- Porosity
- PorosityView
"""

# -------------------------------------------------------------------------------
# Library modules import
# -------------------------------------------------------------------------------

import sys, logging

# -------------------------------------------------------------------------------
# Third-party modules
# -------------------------------------------------------------------------------

from code_saturne.Base.QtCore import *
from code_saturne.Base.QtGui import *
from code_saturne.Base.QtWidgets import *

# -------------------------------------------------------------------------------
# Application modules import
# -------------------------------------------------------------------------------

from code_saturne.model.Common import GuiParam
from code_saturne.Base.QtPage import ComboModel
from code_saturne.Base.QtPage import from_qvariant, to_text_string
from code_saturne.Pages.PorosityForm import Ui_PorosityForm
from code_saturne.model.LocalizationModel import LocalizationModel, Zone
from code_saturne.Pages.QMegEditorView import QMegEditorView
from code_saturne.model.PorosityModel import PorosityModel
from code_saturne.model.NotebookModel import NotebookModel

# -------------------------------------------------------------------------------
# log config
# -------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("PorosityView")
log.setLevel(GuiParam.DEBUG)


# -------------------------------------------------------------------------------
# Main view class
# -------------------------------------------------------------------------------

class PorosityView(QWidget, Ui_PorosityForm):

    def __init__(self, parent=None):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_PorosityForm.__init__(self)
        self.setupUi(self)

        self.case = None
        self.model = None
        self.notebook = None
        self.zone = None

    def setup(self, case, zone_name):
        self.case = case
        self.case.undoStopGlobal()
        self.model = PorosityModel(self.case)
        self.notebook = NotebookModel(self.case)
        localization_model = LocalizationModel("VolumicZone", self.case)
        for zone in localization_model.getZones():
            if zone.getLabel() == zone_name:
                self.zone = zone

        if self.zone.isNatureActivated("porosity"):
            self.setViewFromCase()
        else:  # TODO check if content of tab should still be visible or not
            self.displayDefaultView()
            self.setEnabled(False)

        self.case.undoStartGlobal()

    def setViewFromCase(self):
        # TODO should method be renamed ?
        if self.case.module_name() == 'code_saturne':
            self.modelPorosityType = ComboModel(self.comboBoxType, 3, 1)
            self.modelPorosityType.addItem(self.tr("isotropic"), 'isotropic')
            self.modelPorosityType.addItem(self.tr("anisotropic"), 'anisotropic')
            self.modelPorosityType.addItem(self.tr("integral model"), 'integral')
        else:
            self.modelPorosityType = ComboModel(self.comboBoxType, 1, 1)
            self.modelPorosityType.addItem(self.tr("isotropic"), 'isotropic')
            self.modelPorosityType.disableItem(index=0)
        self.comboBoxType.activated[str].connect(self.slotPorosity)
        self.pushButtonPorosity.clicked.connect(self.slotFormulaPorosity)
        self.selectPorosityZones()

    def selectPorosityZones(self):
        zone_label = self.zone.getLabel()
        zone_id = self.zone.getCodeNumber()

        if hasattr(self, "modelScalars"): del self.modelScalars
        log.debug("slotSelectPorosityZones label %s " % zone_label)
        self.groupBoxType.show()
        self.groupBoxDef.show()

        choice = self.model.getPorosityModel(zone_id)
        self.modelPorosityType.setItem(str_model=choice)

        exp = self.model.getPorosityFormula(zone_id)
        if exp:
            self.pushButtonPorosity.setToolTip(exp)
            self.pushButtonPorosity.setStyleSheet("background-color: green")
        else:
            self.pushButtonPorosity.setStyleSheet("background-color: red")

    @pyqtSlot(str)
    def slotPorosity(self, text):
        """
        Method to call 'getState' with correct arguements for 'rho'
        """
        zone_id = self.zone.getCodeNumber()
        choice = self.modelPorosityType.dicoV2M[str(text)]

        self.model.setPorosityModel(zone_id, choice)

    @pyqtSlot()
    def slotFormulaPorosity(self):
        """
        User formula for density
        """
        zone_label = self.zone.getLabel()
        zone_id = self.zone.getCodeNumber()

        choice = self.model.getPorosityModel(zone_id)
        fname = 'porosity'
        if choice == 'anisotropic':
            fname += '+tensorial_porosity'

        exp, req, sca, sym = self.model.getPorosityFormulaComponents(zone_id)

        if exp == None:
            exp = self.getDefaultPorosityFormula(choice)

        exa = """#example: \n""" + self.model.getDefaultPorosityFormula(choice)

        dialog = QMegEditorView(parent=self,
                                function_type='vol',
                                zone_name=zone_label,
                                variable_name=fname,
                                expression=exp,
                                required=req,
                                symbols=sym,
                                known_fields=sca,
                                examples=exa)

        if dialog.exec_():
            result = dialog.get_result()
            log.debug("slotFormulaPorosity -> %s" % str(result))
            self.model.setPorosityFormula(zone_id, str(result))
            self.pushButtonPorosity.setToolTip(result)
            self.pushButtonPorosity.setStyleSheet("background-color: green")

    def displayDefaultView(self):
        self.modelPorosityType = ComboModel(self.comboBoxType, 1, 1)
        self.modelPorosityType.addItem(self.tr("isotropic"), 'isotropic')
        self.modelPorosityType.setItem(str_model="isotropic")
        self.groupBoxType.show()
        self.groupBoxDef.show()


# -------------------------------------------------------------------------------
# Testing part
# -------------------------------------------------------------------------------


if __name__ == "__main__":
    pass

# -------------------------------------------------------------------------------
# End
# -------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module modifies the run_conf object.
- BatchRunningModel
"""
#-------------------------------------------------------------------------------
# Standard modules import
#-------------------------------------------------------------------------------

import os, sys, types

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

from code_saturne import cs_batch
from code_saturne import cs_run_conf

#-------------------------------------------------------------------------------
# Class BatchRunningModel
#-------------------------------------------------------------------------------

class BatchRunningModel(object):
    """
    This class modifies the job info (in run.cfg)
    """
    def __init__(self, path=None, pkg=None, import_legacy=False):
        """
        Constructor.
        """

        self.pkg = pkg
        if self.pkg == None:
            from code_saturne.cs_package import package
            pkg = package()

        self.run_conf = None
        self.path = path

        # Configuration-based information

        i_c = cs_run_conf.get_install_config_info(self.pkg)

        self.resource_name = cs_run_conf.get_resource_name(i_c)
        self.compute_builds = i_c['compute_builds']

        self.batch = cs_batch.batch(self.pkg)

        # Convert from legacy runcase if not updated yet
        # (delaying application of file changes to save).
        # in this case, the run_conf object is pre-loaded so that
        # the "save" operation can apply the (conversion) changes
        # even when the configuration file has not been loaded.

        self.runcase_path = None
        if self.path:
            if import_legacy and not os.path.isfile(self.path):
                dirname = os.path.dirname(self.path)
                if os.path.basename(dirname) == 'DATA':
                    dirname = os.path.join(os.path.basename(dirname), 'SCRIPTS')
                runcase_path = os.path.join(dirname, 'runcase')
                if os.path.isfile(runcase_path):
                    self.runcase_path = runcase_path

        if self.runcase_path:
            from code_saturne import cs_runcase
            runcase = cs_runcase.runcase(runcase_path,
                                         package=self.pkg)
            sections = runcase.run_conf_sections(resource_name=self.resource_name,
                                                 batch_template=i_c['batch'])

            self.run_conf = cs_run_conf.run_conf(self.path,
                                                 package=self.pkg,
                                                 create_if_missing=True)
            for sn in sections:
                if not sn in self.run_conf.sections:
                    run_conf.sections[sn] = {}
                for kw in sections[sn]:
                    self.run_conf.sections[sn][kw] = sections[sn][kw]

    #---------------------------------------------------------------------------

    def __is_changed__(self):
        """
        Check if values have been changed relative to the initial load.
        """

        if not self.run_conf:
            return False

        changed = False

        if self.job_header_lines_ini != self.job_header_lines:
            changed = True

        if not changed:
            t = ('1', 't', 'true', 'y', 'yes')
            f = ('0', 'f', 'false', 'n', 'no')
            for k in self.run_dict:
                s1 = str(self.run_dict_ini[k]).lower()
                s2 = str(self.run_dict[k]).lower()
                if s1 in f and s2 in f:
                    continue
                elif s1 in t and s2 in t:
                    continue
                elif s1 != s2:
                    changed = True
                    break

        if not changed:
            for k in self.job_dict:
                if str(self.job_dict_ini[k]) != str(self.job_dict[k]):
                    changed = True
                    break

        return changed

    #---------------------------------------------------------------------------

    def __update__(self):
        """
        Update the associated dictionnaries and run_conf object.
        """

        if not self.have_mpi:
            self.job_dict['n_procs'] = None
        if not self.have_openmp:
            self.job_dict['n_threads'] = None

        if not self.run_conf:
            return

        # Run information

        for k in self.run_dict:
            # Special case for "initialize": if False, we mean we do not specify
            # this stage (since the others are not specified either)
            if k == 'initialize':
                if self.run_dict[k] == False:
                    self.run_conf.set('run', 'initialize', None)
                else:
                    self.run_conf.set('run', k, self.run_dict[k])
            else:
                self.run_conf.set('run', k, self.run_dict[k])

        # Resource-related information

        if self.job_header_lines != None:
            self.batch.update_lines(self.job_header_lines)
            self.run_conf.set(self.resource_name, 'job_header',
                              os.linesep.join(self.job_header_lines))

        for k in self.job_dict:
            if self.job_dict[k] == None:
                self.run_conf.set(self.resource_name, k, None)
            else:
                self.run_conf.set(self.resource_name, k, str(self.job_dict[k]))

    #---------------------------------------------------------------------------

    def load(self):
        """
        Load or the associated run_conf object if not already done.
        """

        if self.run_conf:
            return

        # Load or build run configuration

        self.run_conf = cs_run_conf.run_conf(self.path,
                                             package=self.pkg,
                                             create_if_missing=True)

        self.run_dict = {}
        self.job_dict = {}

        # Generic job running information (subset of possible "run" info)

        self.run_dict['id'] = self.run_conf.get('run', 'id')
        self.run_dict['compute_build'] = self.run_conf.get('run', 'compute_build')

        self.run_dict['initialize'] = self.run_conf.get_bool('run', 'initialize')
        self.run_dict['compute'] = None
        self.run_dict['finalize'] = None

        # Resource-specific info (subset of resource-based info, and batch)

        self.job_dict['n_procs'] = self.run_conf.get_int(self.resource_name,
                                                         'n_procs')
        self.job_dict['n_threads'] = self.run_conf.get_int(self.resource_name,
                                                           'n_threads')

        self.job_header_lines = None

        if self.batch.rm_type:
            job_header = self.run_conf.get(self.resource_name, 'job_header')
            if not job_header:
                self.run_conf.rebuild_resource()
                job_header = self.run_conf.get(self.resource_name, 'job_header')
            if job_header != None:
                self.job_header_lines = job_header.split(os.linesep)

        if self.job_header_lines != None:
            self.batch.parse_lines(self.job_header_lines)

        # Save initial values (to determine which are changed)

        self.setup_ini = self.run_dict.get('setup', 'param')

        self.run_dict_ini = {}
        for k in self.run_dict:
            self.run_dict_ini[k] = self.run_dict[k]

        self.job_dict_ini = {}
        for k in self.job_dict:
            self.job_dict_ini[k] = self.job_dict[k]

        self.job_header_lines_ini = None
        if self.job_header_lines:
            self.job_header_lines_ini = list(self.job_header_lines)

        # Fix incompatible options if needed
        # (add entries to dictionnaries so __is_changed__ will detect update)

        if self.run_conf.get_bool('run', 'stage') == False:
            if not self.run_dict_ini['id']:
                self.run_dict_ini['stage'] = False
                self.run_dict['stage'] = None

        # Query info related to compute build

        self.updateComputeBuildInfo(self.run_dict['compute_build'])

    #---------------------------------------------------------------------------

    def updateComputeBuildInfo(self, compute_build=None):
        """
        Update relative to compute build
        """

        pkg_compute = None

        if self.compute_builds != None:
            if not compute_build:
                if len(self.compute_builds) > 0:
                    compute_build = self.compute_builds[0]
            if compute_build in self.compute_builds:
                pkg_compute = self.pkg.get_alternate_version(compute_build)

        if pkg_compute == None:
            pkg_compute = self.pkg

        config_features = pkg_compute.config.features
        if config_features['mpi'] == 'yes':
            self.have_mpi = True
        else:
            self.have_mpi = False
        if config_features['openmp'] == 'yes':
            self.have_openmp = True
        else:
            self.have_openmp = False

        if not self.have_mpi:
            self.job_dict['n_procs'] = None
        if not self.have_openmp:
            self.job_dict['n_threads'] = None

    #---------------------------------------------------------------------------

    def save(self, path=None, param=None, force=True):
        """
        Update the associated run_conf object and save file
        """

        if not self.run_conf:
            if  param and path:
                self.path = path
                if os.path.isfile(path):
                    self.load()
            if not self.run_conf:
                return
        else:
            self.__update__()

        if param != None:
            param = os.path.basename(param)
        if param == 'setup.xml':
            param = None

        self.run_conf.set('setup', 'param', param)

        save_as = False

        if path != None:
            if self.path != None:
                if self.path != path:
                    save_as = True
                else:
                    save_as = False
        else:
            path = self.path

        if save_as:
            self.run_conf.save(path, new=True)

        else:

            if force == False:
                force = self.__is_changed__()

            if force:
                self.run_conf.save(path)

                if self.runcase_path != None:
                    os.remove(self.runcase_path)
                    try:
                        os.remove(os.path.dirname(self.runcase_path))
                    except Exception:
                        pass

        self.path = path

#-------------------------------------------------------------------------------
# End of BatchRunningModel
#-------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

"""
This module define the 'ThermodynamicsSaturationView' page.
This module contains the following classes:
- ThermodynamicsSaturationView
"""

#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------

import os, sys, string, types
import logging

#-------------------------------------------------------------------------------
# EOS
#-------------------------------------------------------------------------------

EOS = 1
try:
   import eosAva
except:
   EOS = 0
else :
   import eosAva


#-------------------------------------------------------------------------------
# Third-party modules
#-------------------------------------------------------------------------------

from code_saturne.Base.QtCore    import *
from code_saturne.Base.QtGui     import *
from code_saturne.Base.QtWidgets import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from code_saturne.model.Common import GuiParam
from code_saturne.Base.QtPage import DoubleValidator, ComboModel
from code_saturne.Base.QtPage import to_text_string
from code_saturne.Pages.ThermodynamicsSaturation import Ui_ThermodynamicsSaturation
from code_saturne.model.ThermodynamicsModel import *
from code_saturne.model.MainFieldsModel import MainFieldsModel
from code_saturne.model.SpeciesModel import SpeciesModel
from code_saturne.model.OutputFieldsModel import OutputFieldsModel
from code_saturne.model.NonCondensableModel import NonCondensableModel
from code_saturne.model.LocalizationModel import LocalizationModel

from code_saturne.Pages.QMegEditorView import QMegEditorView
from code_saturne.model.NotebookModel import NotebookModel
from code_saturne.model.InterfacialEnthalpyModel import InterfacialEnthalpyModel

#-------------------------------------------------------------------------------
# log config
#-------------------------------------------------------------------------------

logging.basicConfig()
log = logging.getLogger("ThermodynamicsFieldView")
log.setLevel(GuiParam.DEBUG)


#-------------------------------------------------------------------------------
# Combo box delegate for the material
#-------------------------------------------------------------------------------

_ok_str  = "background-color: green"
_nok_str = "background-color: red"

#-------------------------------------------------------------------------------
# MainFieldsView class
#-------------------------------------------------------------------------------

class ThermodynamicsSaturationView(QWidget, Ui_ThermodynamicsSaturation):
    """
    Thermodynamics layout.
    """

    def __init__(self, parent = None):
        """
        Constructor
        """
        QWidget.__init__(self, parent)

        Ui_ThermodynamicsSaturation.__init__(self)
        self.setupUi(self)

        self.case      = None
        self.mdl       = None
        self.notebook  = None
        self.zone      = None
        self.zone_name = None
        self.zone_id   = None

    def setup(self,case, zone_name):
        self.case = case
        for zone in LocalizationModel('VolumicZone', self.case).getZones():
            if zone.getLabel() == zone_name:
                self.zone = zone
                self.zone_name = zone.getLabel()
                self.zone_id   = zone.getCodeNumber()
        self.case.undoStopGlobal()

        self.mdl      = ThermodynamicsModel(self.case)
        self.notebook = NotebookModel(self.case)
        self.ncond    = NonCondensableModel(self.case)
        self.interf   = InterfacialEnthalpyModel(self.case)
        self.mfm      = MainFieldsModel(self.case)

        # Set fields string:
        field_ids = self.interf.getEnthalpyCoupleFieldId()
        couple_str = ""
        if field_ids:
            couple_str = self.mfm.getLabel(field_ids[0])
            couple_str += " (liquid)/"
            couple_str += self.mfm.getLabel(field_ids[1])
            couple_str += " (gas)"

        self.lineEditCoupledFields.setText(couple_str)
        self.lineEditCoupledFields.setEnabled(False)

        # Check if user properties:
        self.user_method = False
        if field_ids:
            if self.mdl.getMethod(field_ids[0]) == "user_properties" \
                    and self.mdl.getMethod(field_ids[1]) == "user_properties":
                self.user_method = True

        is_main_zone = (zone_name == "all_cells")
        # Dico
        self.m_out = OutputFieldsModel(self.case)
        self.currentFluid = 0

        # Connections
        self.sat_ppts = {"SaturationTemperature":"Tsat",
                         "d_Tsat_d_P":"dTsatdp",
                         "LatentHeat":"Hlat",
                         "SaturationEnthalpyLiquid":"HsatL",
                         "SaturationEnthalpyGas":"HsatG",
                         "d_Hsat_d_P_Liquid":"dHsatLdp",
                         "d_Hsat_d_P_Gas":"dHsatGdp"}


        for key in self.sat_ppts.keys():
            _k = self.sat_ppts[key]
            _button  = getattr(self, "pushButton" + _k)

            # WARNING: when connecting a pushButton, a 'state' signal
            # is emitted. If you do not provide state to lambda,
            # input will be False/True instead of the variable
            _button.clicked.connect(lambda state, name=key: self.slotFormula(name))


        # load Field

        self.initializeWidget()

        self.case.undoStartGlobal()

    def initializeWidget(self):

        # hide groupBoxConstantProperties
        self.groupBoxEauvap.setVisible(self.user_method)

        if self.user_method:
            for key in self.sat_ppts.keys():
                _k = self.sat_ppts[key]
                _button = getattr(self, "pushButton" + _k)

                exp = self.mdl.getFormula("none", key, self.zone_id)
                if exp:
                    _button.setStyleSheet(_ok_str)
                    _button.setToolTip(exp)
                else:
                    _button.setStyleSheet(_nok_str)

    @pyqtSlot()
    def slotFormula(self, name):
        """
        User formula for a given saturation property.
        """

        exp, req, sca, symbols = self.mdl.getFormulaComponents('none', name, zone=self.zone_id)

        if not exp:
            exp = self.mdl.getDefaultFormula('none', name)

        exa = self.mdl.getExampleFormula('none', name)

        dialog = QMegEditorView(parent        = self,
                                function_type = 'vol',
                                zone_name     = self.zone_name,
                                variable_name = name,
                                expression    = exp,
                                required      = req,
                                symbols       = symbols,
                                known_fields  = sca,
                                examples      = exa)

        if dialog.exec_():
            result = dialog.get_result()
            _k = self.sat_ppts[name]
            _button  = getattr(self, "pushButton" + _k)
            log.debug("slotFormula%s -> %s" % (_k, str(result)))
            self.mdl.setFormula('none', name, result, zone=self.zone_id)
            _button.setStyleSheet(_ok_str)
            _button.setToolTip(result)


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------

